##########################################################################
#
# __init__.py:  defines this directory as the 'tests' package.
#
#  Debian Partial Mirror is a tool to make complete and partial Debian mirrors.
#  See http://projetos.ossystems.com.br/debpartial-mirror/ for more information.
#
# ====================================================================
# Copyright (c) 2002-2005 O.S. Systems.  All rights reserved.
#
# This software is licensed as described in the file COPYING, which
# you should have received as part of this distribution.
#
#########################################################################
# Author: Otavio Salvador <otavio@ossystems.com.br>

import unittest

import TestConfig
import TestDownload
import TestEndToEnd

def suite():
	suite = unittest.TestSuite()
	suite.addTest(TestConfig.suite())
	suite.addTest(TestDownload.suite())
	suite.addTest(TestEndToEnd.suite())
	return suite


if __name__ == '__main__':
	unittest.main(defaultTest='suite')



#!/usr/bin/python
"""
Tool to build partial debian mirrors (and more)
"""
# debpartial-mirror - partial debian mirror package tool
# (c) 2006 Otavio Salvador <otavio@debian.org>
# (c) 2006 Marco Presi <zufus@debian.org>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

# -------
# Imports
# -------


import gtk

from debpartial_mirror_ui import Callbacks
from debpartial_mirror_ui import GDebPartialMirror


if __name__ == "__main__":
    ui = GDebPartialMirror.GDebPartialMirror("/etc/debpartial-mirror.conf")
    gtk.main()




from os import sys

sys.path.append ('../')

from gtk import Window, main, main_quit
from ConfigEditor import *
from debpartial_mirror import Config, Backend

cnf = Config.Config('/etc/debpartial-mirror.conf')

mirrors = []
merges = []

for b in cnf.get_backends()[0]:
    mirrors.append(Backend.MirrorBackend(b.section,cnf))

for b in cnf.get_backends()[1]:
    merges.append(Backend.MirrorBackend(b.section,cnf))


w = Window()
bb = ConfigEditor (cnf, debug = True)
w.add(bb)

w.show_all()
w.connect('destroy', main_quit)
main()



from os import sys

sys.path.append ('../')


from gtk import Window, main, main_quit
from BackendBrowser import BackendBrowser
from debpartial_mirror import Config, Backend

cnf = Config.Config('/etc/debpartial-mirror.conf')

#mirrors = []
#merges = []

#for b in cnf.get_backends()[0]:
#    mirrors.append(Backend.MirrorBackend(b.section,cnf))

#for b in cnf.get_backends()[1]:
#    merges.append(Backend.MirrorBackend(b.section,cnf))


w = Window()
bb = BackendBrowser (cnf, debug = True)
w.add(bb)

w.show_all()
w.connect('destroy', main_quit)
main()



##########################################################################
#
# __init__.py:  defines this directory as the 'tests' package.
#
#  Debian Partial Mirror is a tool to make complete and partial Debian mirrors.
#  See http://projetos.ossystems.com.br/debpartial-mirror/ for more information.
#
# ====================================================================
# Copyright (c) 2002-2005 O.S. Systems.  All rights reserved.
#
# This software is licensed as described in the file COPYING, which
# you should have received as part of this distribution.
#
#########################################################################
# Author: Otavio Salvador <otavio@ossystems.com.br>

from os.path  import abspath, join, dirname

import os
import shutil
import sys
import tempfile
import unittest

class TestBase(unittest.TestCase):
	use_relative_paths = os.environ.has_key("USE_RELATIVE_PATHS")
	TEST_DIR = "tests"

	def setUp(self):
		self._tempDirs = []
		self._tempFiles = []

	def tearDown(self):
		for tempDir in self._tempDirs:
			shutil.rmtree(tempDir)

		for tempFile in self._tempFiles:
			if os.path.exists(tempFile):
				os.remove(tempFile)

	def aux_rel_path(self):
		if "aux" in os.listdir("."):
			return "aux"
		return "%s/aux" % self.TEST_DIR

	def aux_file(self, filename):
		return abspath(os.path.join(self.aux_rel_path(), filename))
		current_dir = abspath(dirname(sys.argv[0]))

		if not current_dir.endswith(self.TEST_DIR):
			current_dir += "/" + self.TEST_DIR

		return join(current_dir, 'aux', filename)

	def createTempDirectory(self):
		if self.use_relative_paths:
			tempDir = tempfile.mkdtemp(prefix="dpmtest", dir=".")
		else:
			tempDir = tempfile.mkdtemp(prefix="dpmtest")
		self._tempDirs.append(tempDir)
		return tempDir

	def createTempFile(self):
		if self.use_relative_paths:
			tempFile = tempfile.mktemp(prefix="dpmtest", dir=".")
		else:
			tempFile = tempfile.mktemp(prefix="dpmtest")
		self._tempFiles.append(tempFile)
		return tempFile



# debpartial_mirror - partial debian mirror package tool
# (c) 2004 Otavio Salvador <otavio@debian.org>, Marco Presi <zufus@debian.org>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA	02111-1307	USA

# TODO:
# Add a control on md5sum to check wich files has to be updated

import os
import re
import time

import apt_pkg

from cdd import Package
from cdd import PackageList
from cdd import FileSystem

from debpartial_mirror import Config
from debpartial_mirror import Download
from debpartial_mirror import Failures


class Dists:
	"""
	This class provides methods to manage dists on partial-mirrors
	"""
	_commandExecutor = os.system  # injection point for unittests

	def __init__ (self, backend):
		self._backend = backend
		self._filesystem = FileSystem.FileSystem(
			backend["mirror_dir"],
			backend["name"],
		)
		self._files = []
		self._already_loaded = False

	def _get_backends_for_names(self, backend_names):
		backends = []
		# Build our list of PackageLists to be used for resolve_depends
		for backend in self._backend.backends:
			if backend._name in backend_names:
				backends.append(backend)
		return backends


	def _resolve_deps_using_backends(self, requiredPackages, backends, architecture):
		# Resolve our dependencies using pkglists
		pkglists = []
		for backend in backends:
			pkglists.append(backend.get_full_binary_list(architecture))
		try:
			requiredPackages.resolve_depends(pkglists, fastFail=False)
		except PackageList.BrokenDependencies, exception:
			userFailure = Failures.DependencyResolutionFailure(exception.broken)
			if self._backend.get_config_with_default("standalone", False):
				raise userFailure
			print userFailure.format(
				prefix="WARNING: Can't resolve dependencies:",
				linePrefix="    "
			)


class MirrorDists(Dists):

	def __init__(self, backend):
		Dists.__init__(self, backend)

		# Partial package lists per architecture
		self.__bin = {}
		self.__source = PackageList.PackageList()

		# Full package lists per architecture
		self.__full_bin	   = {}
		self.__full_source = PackageList.PackageList()

	def _fill_files(self):
		# If we already have it doesn't rerun
		if self._files:
			return

		for component in self._backend["components"]:
			for dist in self._backend["distributions"]:
				dist='dists/'+dist
				for arch in self._backend["architectures"]:
					arch='binary-'+arch
					mirror_name = self._backend["mirror_dir"] +\
								  self._backend["name"]

					self._files.append("%s/%s/%s/Packages.gz" % (
						dist, component, arch
					))
					self._files.append("%s/Release" % dist )
					self._files.append("%s/Release.gpg" % dist )
					if not component.endswith("/debian-installer"):
						# debian-installer components don't have Release files
						self._files.append("%s/%s/%s/Release" % (
							dist, component, arch
						))
			if self._backend["get_sources"] and not component.endswith("/debian-installer"):
				self._files.append("%s/%s/source/Sources.gz" % (
					dist, component
				))

	def get_index(self):
		"""
		Get only index files (on Debian, this mean: Packages.gz
		and Sources.gz)
		"""
		# TODO: Put checking using Release file when available.
		files = []
		self._fill_files()
		for file in self._files:
			if not os.path.basename(file).startswith("Release"):
				files.append(file.split('.gz')[0])
		return files

	def get_binary_list(self, architecture):
		return self.__bin.setdefault(architecture, PackageList.PackageList())

	def get_source_list(self):
		return self.__source

	def get_full_binary_list(self, architecture):
		return self.__full_bin.setdefault(architecture, PackageList.PackageList())

	def get_full_source_list(self):
		return self.__full_source

	def load(self):
		if self._already_loaded:
			return True

		for file in self.get_index():
			distribution = file.split(os.path.sep)[1]

			# Choose object type
			if os.path.basename(file) == 'Packages':
				pkg = Package.Package
				architecture = re.match(r".*/binary-(.*)/.*", file).group(1)
				pkglist = self.get_full_binary_list(architecture)
			elif os.path.basename(file) == 'Sources':
				pkg = Package.SourcePackage
				pkglist = self.__full_source
				architecture = "source"

			# Read release file to determine component
			class ReleaseInfo:
				def __init__(self, distribution, architecture):
					self.distribution = distribution
					self.architecture = architecture
					self.component = None
				def parse(self, section):
					self.component = section["Component"]

			releaseInfo = ReleaseInfo(distribution, architecture)
			release_filename = os.path.join(
				self._filesystem.base(),
				os.path.dirname(file),
				'Release'
			)
			if os.path.exists(release_filename):
				processTagFile(release_filename, releaseInfo.parse)

			# Load file on list
			def addPackage(section):
				package = pkg(section)
				package.releaseInfo = releaseInfo
				name = package['Package']
				if name in pkglist:
					oldver = pkglist[name]['Version']
					newver = package['Version']
					if apt_pkg.version_compare(oldver, newver) < 1:
						pkglist.remove(name)
						pkglist.add(package)
				else:
					pkglist.add(package)

			index_filename = os.path.join(self._filesystem.base(), file)
			if os.path.exists(index_filename):
				processTagFile(index_filename, addPackage)
			else:
				print "Cannot load mirror '%s' due to missing file '%s'" % (
					self._backend["name"], index_filename)
				return False
		self._already_loaded = True
		return True

	def filter(self):
		pkgfilter = []
		try:
			pkgfilter = self._backend['filter']
		except Config.InvalidOption:
			pass

		# to load indexes
		self.load()
		# Apply filter or use as final list
		if pkgfilter:
			for architecture in self._backend["architectures"]:
				self.__bin[architecture] = self.get_full_binary_list(architecture).filter(pkgfilter)
			self.__source = self.__full_source.filter(pkgfilter)
		else:
			self.__bin = self.__full_bin
			self.__source = self.__full_source

	def resolve(self):
		backend_names = [self._backend._name]

		try:
			backend_names = self._backend['resolve_deps_using'] + backend_names
		except Config.InvalidOption:
			pass # Is possible to don't have this option

		for architecture in self._backend["architectures"]:
			self._resolve_deps_using_backends(
				self.get_binary_list(architecture),
				self._get_backends_for_names(backend_names),
				architecture
			)

	def process(self):
		self.filter()
		self.resolve()


	def writeIndexFiles(self):
		indices = _Indices()
		for architecture in self._backend["architectures"]:
			for pkg in self.get_binary_list(architecture).values():
				indices.addPackage(pkg, self._backend, pkg.releaseInfo.distribution)

		for dist in self._backend["distributions"]:
			releaseFile = os.path.join(self._filesystem.base(), "dists", dist, "Release")
			release = tagFileToSectionMapList(releaseFile)[0]
			release["Archive"] = release["Suite"]
			indices.writeFiles(
				self._filesystem,
				dist,
				release,
				self._backend.get_config_with_default("signature_key", None),
				self._commandExecutor,
			)


class RemoteDists (MirrorDists):
	"""
	This class provides methods to fill dists dir downloading remote files
	"""

	def update (self):
		""" Get only files that need updates """
		self._fill_files()
		download = Download.Download(name="Dist_" + self._backend["name"])
		for file in self._files:
			self._filesystem.mkdir(os.path.dirname(file))
			server = "%s/%s" % (self._backend["server"], file)
			filename = "%s/%s" % (self._filesystem.base(), file)
			download.get(server, filename)

		download.wait_mine()

		for file in self._files:
			if not os.path.basename(file).startswith("Release"):
				try:
					self._filesystem.uncompress(file)
				except IOError:
					return False


class LocalDists (MirrorDists):
	"""
	This class provides methods to fill dists dir downloading local files
	"""

	def update (self):
		""" Get only files that need updates """
		self._fill_files()
		for server, filename, dirname in self._files:
			orig, filename = file
			self._filesystem.mkdir(dirname)
			os.link (orig.split('file://')[1], filename)


class MergeDists (Dists):
	"""
	This class provides methods to fill dists dir when merging backends
	"""
	def __init__(self, backend):
		Dists.__init__(self, backend)

		self.__mirrors = self._get_backends_for_names(self._backend['backends'])

	def merge(self):
		indices = _Indices()
		# Fill package lists
		for architecture in self._backend["architectures"]:
			for mirror in self.__mirrors:
				for pkg in self.get_packages_for_mirror(mirror, architecture).values():
					indices.addPackage(pkg, mirror, self._backend['name'])

		# Write package lists and release files
		release = {"Archive" : self._backend._name}
		for key in ['origin', 'label', 'description', 'suite', 'codename', 'version']:
			fieldname = key[0].upper() + key[1:]
			if self._backend.has_key(key):
				release[fieldname] = self._backend[key]
			else:
				release[fieldname] = "DebPartialMirror"

		indices.writeFiles(
			self._filesystem,
			self._backend._name,
			release,
			self._backend.get_config_with_default("signature_key", None),
			self._commandExecutor,
		)

	def get_mirrors (self):
		return self.__mirrors

	def get_packages_for_mirror(self, mirror, architecture):
		pkgfilter = []
		try:
			pkgfilter = self._backend['filter_%s' % mirror._name]
		except Config.InvalidOption:
			return mirror.get_binary_list(architecture)
		packages = mirror.get_binary_list(architecture).filter(pkgfilter)
		self._resolve_deps_using_backends(packages, self.__mirrors, architecture)
		return packages


class _Indices:
	def __init__(self):
		self._indexByFilename = {}

	def addPackage(self, pkg, mirror, dist):
		dist = "dists/" + dist
		component = pkg.releaseInfo.component or pkg['Filename'].split("/")[1]
		if pkg['Filename'].endswith("udeb"):
			component += "/debian-installer"
		tmp = pkg['Filename'].split("_")
		tmp = tmp[len(tmp) -1]
		arch = tmp.split(".")[0]

		if pkg['Filename'].endswith("deb"):
			if arch == "all":
				for arch in mirror["architectures"]:
					self._addBinaryPackage(dist, component, arch, pkg)
			else:
				self._addBinaryPackage(dist, component, arch, pkg)

		if pkg['Filename'].endswith(".dsc") or pkg['Filename'].endswith(".gz"):
			self._addSourcePackage(dist, component, pkg)

	def writeFiles(self, filesystem, dist, release, signatureKey, commandExecutor):
		def addUnique(collection, item):
			if item not in collection:
				collection.append(item)

		components = []
		architectures = []
		files = []

		def writeReleaseFields(out, *fieldnames):
			for fieldname in fieldnames:
				if fieldname in release:
					out.write("%s: %s\n" % (fieldname, release[fieldname]))

		for index in self._indexByFilename.values():
			if not filesystem.exists(index.getDirectory()):
				filesystem.mkdir(index.getDirectory())

			if index.filename.endswith("Packages"):
				addUnique(architectures, index.architecture)

				# Write Release
				if index.component.endswith("/debian-installer"):
					# debian-installer components don't have Release files
					addUnique(components, "/".join(index.component.split("/")[:-1]))
				else:
					addUnique(components, index.component)

					file = os.path.join(index.getDirectory(), "Release")
					files.append("/".join(file.split("/")[2:])) #??
					out = open(os.path.join(filesystem.base(), file), "w+")
					try:
						writeReleaseFields(out, "Archive", "Version")
						out.write("Component: %s\n" % (index.component))
						writeReleaseFields(out, "Origin", "Label")
						out.write("Architecture: %s\n" % (index.architecture))
					finally:
						out.close()

			# Write index
			out = open(os.path.join(filesystem.base(), index.filename), "w+")
			try:
				for pkg in sorted(index.packageList.values(), lambda x,y : cmp(x["Package"], y["Package"])):
					out.write(pkg.dump() + "\n")
			finally:
				out.close()
			filesystem.compress(index.filename)
			filename = "/".join(index.filename.split("/")[2:])
			files.append(filename)
			files.append(filename + ".gz")


		out = open(os.path.join(filesystem.base(), "dists/" + dist + "/Release"), "w+")
		try:
			writeReleaseFields(out, "Origin", "Label", "Suite", "Version", "Codename")
			out.write("Date: %s\n" % (
				time.strftime("%a, %d %b %Y %H:%M:%S UTC", time.gmtime(time.time()))
			))
			out.write("Architectures: %s\n" % (" ".join(architectures)))
			out.write("Components: %s\n" % (" ".join(components)))
			out.write("Description: %s\n" % (release['Description']))

			out.write("MD5Sum:\n")
			for filename in files:
				fullpath = "dists/%s/%s" % (dist, filename)
				out.write(" %s %8d %s\n" % (
					filesystem.md5sum(fullpath),
					filesystem.size(fullpath),
					filename,
				))

			out.write("SHA1:\n")
			for filename in files:
				fullpath = "dists/%s/%s" % (dist, filename)
				out.write(" %s %8d %s\n" % (
					filesystem.sha1sum(fullpath),
					filesystem.size(fullpath),
					filename,
				))
		finally:
			out.close()

		if signatureKey:
			releaseFile = os.path.join(filesystem.base(), "dists", dist, "Release")
			keySelector = ""
			if signatureKey != "default":
				keySelector = "-u %s" % signatureKey
			rc = commandExecutor("gpg -abs %s -o - %s > %s.gpg" % (
				keySelector,
				releaseFile,
				releaseFile
			))
			if rc != 0:
				raise IOError("gpg command returned %s", rc)


	def _addBinaryPackage(self, dist, component, architecture, package):
		filename = "%s/%s/binary-%s/Packages" % (dist, component, architecture)
		index = self._indexByFilename.setdefault(filename, _Index(filename, component, architecture))
		index.addPackage(package)

	def _addSourcePackage(self, dist, component, package):
		filename = "%s/%s/source/Packages" % (dist, component)
		index = self._indexByFilename.setdefault(filename, _Index(filename, component, "source"))
		index.addPackage(package)


class _Index:
	def __init__(self, filename, component, architecture):
		self.filename = filename
		self.component = component
		self.architecture= architecture
		self.packageList = PackageList.PackageList()

	def getDirectory(self):
		return os.path.dirname(self.filename)

	def addPackage(self, package):
		if not self.packageList.has_key(package["Package"]):
			self.packageList.add(package)


def processTagFile(filename, sectionHandler):
	parse_in = open(filename, "r")
	try:
		for section in apt_pkg.TagFile(parse_in):
			sectionHandler(section)
	finally:
		parse_in.close()


def tagFileToSectionMapList(filename):
	sections = []

	def handleSection(section):
		tags = {}
		for key in section.keys():
			if key:
				tags[key] = section[key]
		sections.append(tags)

	processTagFile(filename, handleSection)
	return sections



#!/usr/bin/python
import unittest
import sys

sys.path.append('lib')

import __init__

if not unittest.TextTestRunner(verbosity=2).\
                               run(__init__.suite()).wasSuccessful():
    sys.exit(1)



# debpartial_mirror - partial debian mirror package tool
# (c) 2004 Otavio Salvador <otavio@debian.org>, Marco Presi <zufus@debian.org>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA	02111-1307	USA

import os
import sys

import apt_pkg

from cdd import FileSystem

from debpartial_mirror import Download


class Pool:
	"""
	This class provides methods to manage pool dirs into the
	partial-mirror.
	"""
	def __init__(self, backend):
		self._backend = backend
		self._got_files = []
		self._containsForeignPackages = False
		self._filesystem = FileSystem.FileSystem(
			backend["mirror_dir"],
			backend["name"]
		)

	def _process_package(self, pkg):
		"""
		This method is called for each package and should be override
		in each child class.
		"""
		raise NotImplemented

	def _calc_need_files(self):
		"""
		This method is called to build the list of need files to
		permit to us to clean up the mirror. If you need to handle in
		a special way to calc it, you're free to override this method.
		"""
		if not len(self._got_files):
			self._backend.process()
			need_files = []
			for architecture in self._backend["architectures"]:
				need_files.extend([pkg['Filename'] for pkg in self._backend.get_binary_list(architecture).values()])
		else:
			need_files = self._got_files

		return need_files

	def _link_internal(self, pkg, mirror):
		filename = pkg['Filename']
		self._filesystem.mkdir(os.path.dirname(filename))
		src = self._backend["mirror_dir"] + "/" + mirror["name"] + "/" + filename
		dst = os.path.join(self._filesystem.base(), filename)
		if not os.path.exists(dst):
			try:
				print "Linking [%s] %s" % (mirror["name"],  filename)
				os.link (src, dst)
			except OSError, msg:
				print "Error while linking a file:"
				print " [%s] %s" % (mirror["name"], filename)
				print " System message: %s" % msg
				sys.exit(1) #TODO: fixme
		self._got_files.append(filename)

	def upgrade(self):
		"""
		Manage the pool upgrade process
		TODO: Handle source packages.
		"""
		def processPackage(pkg, architecture):
			foundList =  self._backend.get_binary_list(architecture).whereis(pkg["Package"])
			if foundList in  (self._backend.get_binary_list(architecture), self._backend.get_full_binary_list(architecture)):
				self._process_package(pkg)
			else: # required dependency from another mirror
				for backend in self._backend.backends:
					if backend.get_full_binary_list(architecture) == foundList:
						break
				else:
					raise RuntimeError("No backend found for %s", pkg["Package"])

				self._link_internal(pkg, backend)
				self._containsForeignPackages = True

		for architecture in self._backend["architectures"]:
			for pkg in self._backend.get_binary_list(architecture).values():
				filename = pkg['Filename']
				if not self._filesystem.exists(filename):
					processPackage(pkg, architecture)
				elif self._filesystem.md5sum(filename) == pkg['MD5sum']:
					self._got_files.append(filename)
				else:
					print 'Removing corrupted package', pkg['Package']
					self._filesystem.remove(filename)
					processPackage(pkg, architecture)


	def clean (self):
		"""
		Clean old files
		"""
		need_files = self._calc_need_files()

		# Walk in all debian related files of mirror
		for current in self._filesystem.match("^.+\.(dsc|(diff|tar)\.gz|deb|udeb)$"):
			if current not in need_files:
				print "Removing unneeded file: %s" % current
				self._filesystem.remove(current)
				try:
					self._filesystem.rmdir(os.path.dirname(current), True)
				except OSError:
					pass

	def containsForeignPackages(self):
		return self._containsForeignPackages


class RemotePool (Pool):
	"""
	This class provides methods to fill pool dir downloading remote files
	"""
	def __init__ (self, backend):
		Pool.__init__(self, backend) # Call parent constructor

		self.download = Download.Download(name = "Pool_" + backend["name"]) # Our Download manager

	def _process_package (self, pkg):
		self._filesystem.mkdir(os.path.dirname(pkg['Filename']))

		self.download.get(
			os.path.join(self._backend["server"], pkg['Filename']),
			os.path.join(self._filesystem.base(), pkg['Filename'])
		)
		self._got_files.append(pkg['Filename'])

	def upgrade (self):
		Pool.upgrade(self) # Call parent method
		self.download.wait_mine() # Wait until our downloads finish


class LocalPool (Pool):
	"""
	This class provides methods to fill pool dir linking from local files
	"""
	def _process_package (self):
		self._filesystem.mkdir(os.path.dirname(pkg['Filename']))
		os.link(
			self._backend["server"].split('file://')[1] + pkg,
			self._local + pkg
		)
		self._got_files.append(pkg['Filename'])


class MergePool (Pool):
	"""
	This class provides methods to fill pool dir mergin other pools
	"""

	def _calc_need_files(self):
		need_files = []
		for architecture in self._backend["architectures"]:
			for mirror in self._backend.get_mirrors():
				for pkg in self._backend.get_packages_for_mirror(mirror, architecture).values():
					need_files.append(pkg['Filename'])
		return need_files

	def merge (self):
		for architecture in self._backend["architectures"]:
			for mirror in self._backend.get_mirrors():
				for pkg in self._backend.get_packages_for_mirror(mirror, architecture).values():
					self._link_internal(pkg, mirror)




# debpartial_mirror_ui - User Interface for partial debian mirror package tool
# (c) 2006 Otavio Salvador <otavio@debian.org>
# (c) 2006 Marco Presi <zufus@debian.org>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


from gtk import VBox, HBox, Notebook, VPaned, Label, \
     TreeView, TreeIter, TreeViewColumn, TreeStore, \
     CellRendererText, Frame, SHADOW_IN, SELECTION_SINGLE

import gobject

import ConfigParser

from debpartial_mirror import Config


class PropertyEditor (TreeView):
    """ A TreeView with property/value pairs """


    __gsignals__ = dict(property_changed=(gobject.SIGNAL_RUN_FIRST,
                                          gobject.TYPE_NONE,
                                          (gobject.TYPE_STRING,)))


    def __init__ (self, cnf):
        super (PropertyEditor, self).__init__()

        self.cnf = cnf
        self.set_property ('visible', True)
        self.set_property ('can_focus', True)
        self.set_property ('headers_visible', True)
        self.set_property ('enable_search', True)
        self.set_property ('reorderable', True)
        self.set_property ('rules_hint',False)
        self.set_property ('fixed_height_mode', False)
        self.set_property ('hover_selection', False)
        self.set_property ('hover_expand', False)

        (self.COLUMN_PROP,
         self.COLUMN_VALUE) = range(2)


        self.model = TreeStore (gobject.TYPE_STRING,
                                gobject.TYPE_STRING)
        
        renderer = CellRendererText ()
        renderer.set_property ("xalign", 0.0)

        column = TreeViewColumn('Key',
                                renderer,
                                text=self.COLUMN_PROP)

        column.set_clickable(False)
        column.set_sort_column_id(self.COLUMN_PROP)
        self.append_column(column)

        column = TreeViewColumn('Value',
                                renderer,
                                text=self.COLUMN_VALUE)

        column.set_clickable(False)
        column.set_sort_column_id(self.COLUMN_VALUE)
        self.append_column(column)

        select = self.get_selection ()
        select.set_mode (SELECTION_SINGLE)
        select.connect ('changed', self.property_selected_cb)


        self.expand_all()
        self.set_rules_hint(True)
        self.show_all()

    def property_selected_cb (self, selection):
        (model, iter) = selection.get_selected ()
        if isinstance (iter, TreeIter): 
            print "%s selected\n" % model.get_value (iter, self.COLUMN_PROP)
            self.emit ('property_changed', model.get_value (iter, self.COLUMN_PROP))



    def update_property_editor (self, b_name):
        """ A method to update the propety/value pair
        after that a backend is selected """

        print "update_property_editor: %s" % b_name


        try:
            options = self.cnf.options(b_name)
            self.model.clear ()
            for b in options:
                print "Adding %s" %  b
                self.iter = self.model.append (None)
                self.model.set (self.iter,
                                self.COLUMN_PROP, b,
                                self.COLUMN_VALUE, self.cnf.get_option(b, b_name))
                
        except ConfigParser.NoSectionError:
            # If the selected row was not a backend, we simply ignore it
            if b_name == "<b>Global</b>":
                self.update_property_editor ('GLOBAL')

        self.set_model (self.model)





class PropertyDescriptor (HBox):
    """A display widget that shows a description of the
    selected property """


    prop = {'mirror_dir': 'Filesystem path where the partial mirror is built',
            'components': 'Specify which archives sections should be mirrored',
            'get_packages':'Add description',
            'distributions': 'list of distributions (stable, testomg,..) to mirror',
            'get_provides': 'Add description',
            'get_recommends': 'Add description',
            'architectures': 'list of architectures to mirrors',
            'get_suggests': 'Add description',
            'get_sources': 'Add description'}


    def __init__ (self, debug=False):
        super (PropertyDescriptor, self).__init__ ()

        self._debug_mode = debug

        #self._default_msg = Label("Select a config file section.")
        self._prop_des    = ""
        self._prop_name   = ""
        
        self.prop_name_label = Label ("Name:")
        self.prop_desc_label = Label ("Description:")

        self.prop_name_text = Label ()
        self.prop_desc_text = Label ()
        
        
        label_box = VBox ()
        desc_box  = VBox ()

        label_box.pack_start (self.prop_name_label)
        label_box.pack_start (self.prop_desc_label)

        desc_box.pack_start (self.prop_name_text)
        desc_box.pack_start (self.prop_desc_text)
        
        self.pack_start (label_box)
        self.pack_start (desc_box)

        self.show_all ()

    def update_property_descriptor (self, w, prop_name):
        """ Modify the labels to describe the property selected in
        the configuration editor"""


        print "update_property_descriptor: %s" % prop_name

        try:
            prop_desc = self.prop[prop_name]
            self.prop_name_text.set_text ("%s" % prop_name)
            self.prop_desc_text.set_text ("%s" % prop_desc)

        except KeyError, e:
            print "Errore %s" % e



class ConfigEditor (VPaned):
    """A widget to read/edit the configuration files """

    def __init__ (self, cnf, debug=False):
        super (ConfigEditor, self).__init__()

        self._debug_mode = debug
        
        self.l = PropertyDescriptor ()
        self.p = PropertyEditor (cnf)

        self.p.update_property_editor ('GLOBAL')
        #self.update_label (self.l)

        pframe = Frame ()
        lframe = Frame ()

        pframe.set_shadow_type (SHADOW_IN)
        lframe.set_shadow_type (SHADOW_IN)

        pframe.add (self.p)
        lframe.add (self.l)
        
        self.add1 (pframe)
        self.add2 (lframe)

        self.p.connect ('property_changed', self.l.update_property_descriptor)


    
        self.show_all ()


    def update_property_editor (self, b_name):
        self.p.update_property_editor (b_name)

    def update_label (self, l):
        """ Change the label text """

        l.set_text ('Select a config file section.')






import gzip
import hashlib
import os
import re
import StringIO

import apt_pkg


class AbstractRepository:
	def __init__(self, rawIndex=False, gzIndex=True, multi=False, releaseFields=None, missingReleaseFields=()):
		self._hasRawIndex = rawIndex
		self._hasGzIndex = gzIndex
		self._multi = multi

		self._releaseFields = {
			"Origin" : "Repository Simulator",
			"Label" : "Simulator Label",
			"Suite" : "Simulator Suite",
			"Version" : "Simulator Version",
			"Date" : "Simulator Date",
			"Description" : "Simulator Description",
		}
		for fieldname in missingReleaseFields:
			self._releaseFields.pop(fieldname, None)

		if releaseFields is not None:
			self._releaseFields.update(releaseFields)

		self._packageLists = {}
		self._installerPackageLists = {}
		self._sourcePackageLists = {}

	def getReleaseFields(self):
		return self._releaseFields

	def isMulti(self):
		return self._multi

	def createPackageList(self, distribution, component, architecture):
		packageList = _PackageList()
		self._packageLists[(distribution, component, architecture)] = packageList
		return packageList

	def createInstallerPackageList(self, distribution, component, architecture):
		packageList = _InstallerPackageList()
		self._installerPackageLists[(distribution, component, architecture)] = packageList
		return packageList

	def createSourcePackageList(self, distribution, component):
		packageList = _SourcePackageList()
		self._sourcePackageLists[(distribution, component)] = packageList
		return packageList

	def writeToFilesystem(self, baseDir):
		for filename, contents in self.implCalculateFiles().iteritems():
			pathname = os.path.join(baseDir, filename)
			dirname = os.path.dirname(pathname)
			if not os.path.exists(dirname):
				os.makedirs(dirname)
			f = open(pathname, "w")
			try:
				f.write(contents)
			finally:
				f.close()

	def _addIndexFile(self, files, filename, contents):
		if self._hasRawIndex:
			files[filename] = contents
		if 	self._hasGzIndex:
			buf = StringIO.StringIO()
			compressor = gzip.GzipFile(mode="wb", fileobj=buf)
			compressor.write(contents)
			compressor.close()
			files[filename + ".gz"] = buf.getvalue()
			buf.close()
		return files

	def _addReleaseFile(self, files, path, distribution, component, architecture):
		fields = []
		for key, value in (
			("Archive", distribution),
			("Component", component),
			("Architecture", architecture),
		):
			fields.append("%s: %s" % (key, value))
		files[os.path.join(path, "Release")] = "\n".join(fields)

	def implCalculateFiles(self):
		raise NotImplementedError()



class AutomaticRepository(AbstractRepository):
	def readFromFilesystem(self, baseDir):
		self._baseDir = baseDir
		distDir, dists, _ = os.walk(os.path.join(baseDir, "dists")).next()
		def loadPackages(packageListFactory, key, indexFile):
			packageList = packageListFactory(*key)
			if os.path.exists(indexFile):
				for packageInfo in parseTagFile(indexFile):
					packageList.addPackage(packageInfo["Package"], **packageInfo)


		for dist in dists:
			releaseFile = os.path.join(distDir, dist, "Release")
			if not os.path.exists(releaseFile):
				print "%s not found" % releaseFile
				continue
			release = parseTagFile(releaseFile)[0]
			for field in self._releaseFields.keys():
				self._releaseFields[field] = release.get(field, "")

			for component in release["Components"].split(" "):
				for architecture in release["Architectures"].split(" "):
					loadPackages(
						self.createPackageList,
						(dist, component, architecture),
						os.path.join(distDir, dist, component, "binary-%s" % architecture , "Packages")
					)

					loadPackages(
						self.createInstallerPackageList,
						(dist, component, architecture),
						os.path.join(distDir, dist, component, "debian-installer", "binary-%s" % architecture , "Packages")
					)

				loadPackages(
					self.createSourcePackageList,
					(dist, component),
					os.path.join(distDir, dist, component, "source", "Sources")
				)


	def getPackage(self, name, distribution, component, architecture):
		packageList = self._packageLists.get((distribution, component, architecture))
		return packageList.getPackage(name)

	def getPackageNamesForPresentFiles(self):
		""" Obtain package names corresponding to package files present (for cleanup tests) """
		packageNames = []
		packageRe = re.compile("^([^_]+)_(.*)\.(dsc|(diff|tar)\.gz|deb|udeb)")
		for dirpath, dirname, filenames in os.walk(self._baseDir):
			for filename in filenames:
				match = packageRe.match(filename)
				if match:
					packageNames.append(match.group(1))
		return packageNames

	def getAvailablePackageNames(self, distribution, component, architecture):
		""" Obtain package names that are installable (in index files and package files) """
		return self._getAvailablePackageNames(
			self._packageLists,
			(distribution, component, architecture),
			self._collectSingleFilePackageNames,
		)

	def getAvailableInstallerPackageNames(self, distribution, component, architecture):
		return self._getAvailablePackageNames(
			self._installerPackageLists,
			(distribution, component, architecture),
			self._collectSingleFilePackageNames,
		)

	def getAvailableVersions(self, packageName, distribution, component, architecture):
		def collectSingleFilePackageVersion(availableVersions, package):
			if self._packageFileExists(package):
				availableVersions.append(package["Version"])

		versions = self._getAvailablePackageNames(
			self._packageLists,
			(distribution, component, architecture),
			collectSingleFilePackageVersion,
		)
		versions.sort()
		return tuple(versions)

	def _collectSingleFilePackageNames(self, availableNames, package):
		if self._packageFileExists(package):
			availableNames.append(package["Package"])

	def getAvailableSourcePackageNames(self, distribution, component):
		def collector(availableNames, package):
			for filename in package["filenames"]:
				if os.path.exists(os.path.join(self._baseDir, package["Directory"], filename)):
					availableNames.append(package["Package"])

		return self._getAvailablePackageNames(
			self._sourcePackageLists,
			(distribution, component),
			collector,
		)

	def _packageFileExists(self, package):
		filename = os.path.join(self._baseDir, package["Filename"])
		exists = os.path.exists(filename)
		if not exists:
			print "*** %s not found" % filename
		return exists

	def _getAvailablePackageNames(self, packageLists, key, collector):
		packageList = packageLists.get(key)
		if packageList is None:
			return []

		availableNames = []
		for package in packageList.iterfields():
			collector(availableNames, package)
		return availableNames


	def _collectPackageFiles(self, files, filenameGenerator):
		distContents = {}

		def process(packageLists, extraPathElement, generateReleaseFile):
			for (distribution, component, architecture), packageList in packageLists.iteritems():
				distComponents, distArchitectures = distContents.setdefault(distribution, ([], []))
				distComponents.append(component)
				distArchitectures.append(architecture)

				def generateFilename(binaryPackage, filename):
					return filenameGenerator(distribution, component, architecture, binaryPackage, filename)

				indexPath = os.path.join(
					"dists", distribution, component, extraPathElement,
					"binary-%s" % architecture
				)

				self._addIndexFile(
					files,
					os.path.join(indexPath, "Packages"),
					packageList.calculateIndexFile(architecture, generateFilename),
				)

				if generateReleaseFile:
					self._addReleaseFile(files, indexPath, distribution, component, architecture)


		process(self._packageLists, "", generateReleaseFile=True)
		process(self._installerPackageLists, "debian-installer", generateReleaseFile=False)

		for (distribution, (components, architectures)) in distContents.iteritems():
			parts = []
			for field in ("Origin", "Label", "Suite", "Version", "Date", "Description"):
				if field in self._releaseFields:
					parts.append("%s: %s" % (field, self._releaseFields[field]))
			parts.append("Codename: %s" % distribution)
			parts.append("Architectures: %s" % " ".join(architectures))
			parts.append("Components: %s" % " ".join(components))

			files[os.path.join("dists", distribution, "Release")] = "\n".join(parts)


	def _collectSourcePackageFiles(self, files, directoryGenerator):
		for (distribution, component), packageList in self._sourcePackageLists.iteritems():
			def generateDirectory(sourcePackage):
				return directoryGenerator(distribution, component, sourcePackage)

			indexPath = os.path.join("dists", distribution, component, "source")
			self._addIndexFile(
				files,
				os.path.join(indexPath, "Sources"),
				packageList.calculateIndexFile(files, generateDirectory)
			)
			self._addReleaseFile(files, indexPath, distribution, component, "source")



class PooledRepository(AutomaticRepository):
	def implCalculateFiles(self):
		files = {}
		def generatePoolFilename(distribution, component, architecture, binaryPackage, filename):
			fullname = os.path.join("pool", component, filename[0], filename)
			files[fullname] = "simulated contents of %s" % filename
			return fullname

		def generateSourceDirectory(distribution, component, sourcePackage):
			return os.path.join("dists", distribution, component, "source", sourcePackage["Section"])

		self._collectPackageFiles(files, generatePoolFilename)
		self._collectSourcePackageFiles(files, generateSourceDirectory)
		return files



class NonPooledRepository(AutomaticRepository):
	def implCalculateFiles(self):
		files = {}
		def generateFilename(distribution, component, architecture, binaryPackage, filename):
			fullname = os.path.join(
				"dists", distribution, component, "binary-%s" % architecture, binaryPackage["Section"], filename
			)
			files[fullname] = "simulated contents of %s" % filename
			return fullname

		def generateSourceDirectory(distribution, component, sourcePackage):
			return os.path.join("dists", distribution, component, "source", sourcePackage["Section"])

		self._collectPackageFiles(files, generateFilename)
		self._collectSourcePackageFiles(files, generateSourceDirectory)
		return files


class TrivialRepository(AbstractRepository):
	pass


class _PackageList:
	def __init__(self):
		self._packages = []
		self._packagesByName = {}

	def addPackage(self, name, **kw):
		nameParts = name.split("_")
		if len(nameParts) > 1:
			name = nameParts[0]
			defaultVersion = nameParts[1]
		else:
			defaultVersion ="1.0"

		fields = {
			"Version" : defaultVersion,
			"Priority" : "optional",
			"Architecture" : "any",
			"Section" : "python",
		}
		fields.update(kw)
		self._packages.append((name, fields))
		self._packagesByName.setdefault(name, {})[fields["Version"]] = fields

	def calculateIndexFile(self, defaultArchitecture, filenameGenerator):
		lines = []
		for packageName, fields in self._packages:
			architecture = fields["Architecture"]
			if architecture != "all":
				architecture = defaultArchitecture
			lines.append("Package: %s" % packageName)
			lines.append("Architecture: %s" % architecture)
			lines.append("Filename: %s" % filenameGenerator(fields,
				"%s_%s_%s.%s" % (
					packageName, fields["Version"], architecture, self.getExtension()
			)))
			for fieldName, fieldValue in fields.iteritems():
				lines.append("%s: %s" % (fieldName, fieldValue))
			lines.append("")

		return "\n".join(lines)

	def getExtension(self):
		return "deb"

	def getPackage(self, packageName, version=None):
		packages = self._packagesByName[packageName]
		if len(packages) == 1 and version is None:
			return packages.values()[0]
		if version is None:
			raise RuntimeError("Multiple versions exist for '%s' but no version spectified" % packageName)

		return packages[version]

	def iterfields(self):
		for name, fields in self._packages:
			yield fields

class _InstallerPackageList(_PackageList):
	def getExtension(self):
		return "udeb"


class _SourcePackageList:
	def __init__(self):
		self._packages = {}

	def addPackage(self, name, **kw):
		fields = {
			"Version" : "1.0-1",
			"Section" : "python",
			"Format" : "1.0",
		}
		fields.update(kw)
		fileInfo = kw.pop("Files", None)
		if fileInfo is not None:
			filenames = []
			for line in fileInfo.split("\n"):
				md5sum, size, filename = line.strip().split(" ")
				filenames.append(filename)
			fields["filenames"] = filenames

		self._packages[name] = fields

	def calculateIndexFile(self, files, directoryGenerator):
		lines = []

		def addFile(filename, directory):
			contents = "simulated contents of %s" % filename
			files[os.path.join(directory, filename)] = contents
			return " %s %s %s" % (
				hashlib.md5(contents).hexdigest(),
				len(contents),
				filename,
			)

		for packageName, fields in self._packages.iteritems():
			lines.append("Package: %s" % packageName)
			directory = directoryGenerator(fields)
			fields["Directory"] = directory

			for fieldName, fieldValue in fields.iteritems():
				lines.append("%s: %s" % (fieldName, fieldValue))

			lines.append("Files:")
			fullVersion = fields["Version"]
			upstreamVersion = fullVersion.split("-")[0]
			filenames = []
			for suffix in (
				"%s.dsc" % fullVersion,
				"%s.orig.tar.gz" % upstreamVersion,
				"%s.diff.gz" % fullVersion,
			):
				filename="%s_%s" % (packageName, suffix)
				lines.append(addFile(filename, directory))
				filenames.append(filename)
			lines.append("")
			fields["filenames"] = filenames

		return "\n".join(lines)


def parseTagFile(filename):
	sectionMaps = []
	f = open(filename, "r")
	try:
		for section in apt_pkg.TagFile(f):
			m = {}
			for key in section.keys():
				m[key] = section[key]
			sectionMaps.append(m)
		return sectionMaps
	finally:
		f.close()




# debpartial-mirror - partial debian mirror package tool
# (c) 2004 Otavio Salvador <otavio@debian.org>, Nat Budin <natb@brandeis.edu>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA	02111-1307	USA

import unittest
import threading
import logging
import SimpleHTTPServer
import BaseHTTPServer

import os
import sys

from debpartial_mirror import Download
from TestBase import TestBase


# Let us make  our server quiet! ;-)
class MySimpleHTTPRequestHandler(SimpleHTTPServer.SimpleHTTPRequestHandler):
	def log_message(self, format, *args):
		pass # don't output


# Our builtin simple HTTP server
class ServerThread(threading.Thread):
	def __init__(self,
		port,
		HandlerClass = MySimpleHTTPRequestHandler,
		ServerClass = BaseHTTPServer.HTTPServer,
		protocol="HTTP/1.1"):

		server_address = ('', port)
		HandlerClass.protocol_version = protocol
		self.httpd = ServerClass (server_address, HandlerClass)
		threading.Thread.__init__(self)

	def run(self):
		self.httpd.serve_forever()


class DownloadTests(TestBase):
	size_list = (1000, 5000, 10000)
	running_servers = []
	port = 80

	def setUp (self):
		os.environ["http_proxy"] = ""
		self.__createFiles()

	def tearDown(self):
		self.__removeFiles()

	def __start_http(self):
		while True:
			try:
				httpServer = ServerThread(self.port)
				httpServer.setDaemon(True)
				httpServer.start()
				break
			except:
				self.port += 1
				pass
		print "running on port ",self.port
		self.running_servers.append(self.port)

	def __get_http_ports(self):
		return self.running_servers

	def __createFile (self, name, size):
		fname = self.aux_file(name)
		cmd = "dd if=/dev/zero of=%s bs=1024 count=%s 2>/dev/null" % (fname, size)
		self.failUnlessEqual(os.system(cmd), 0,
							 'Failed to create input test file.')

	def __createFiles (self):
		for size in self.size_list:
			fname = "pkg-example_1.%d_all.deb" % size
			self.__createFile(fname, size)

	def __removeFile (self, name):
		if os.path.exists(self.aux_file(name)):
			os.remove (self.aux_file(name))
		if os.path.exists(name):
			os.remove (name)

	def __removeFiles (self):
		for size in self.size_list:
			fname = "pkg-example_1.%d_all.deb" % size
			self.__removeFile (fname)

	def test_1download_without_server(self):
		"""Download: Try to grab a file while HTTPserver is down."""
		d = Download.Download(info="quiet", name="Test1")
		size = self.size_list[0]

		filename = "pkg-example_1.%d_all.deb" % size
		self.__createFile (filename, size)
		d.get('http://invalid/%s' % filename, filename)
		d.wait_mine()
		self.failIf(os.path.exists(filename),
					"Created (empty) downloaded file even if HTTP server was down.")

		del d
		self.__removeFile(filename)

	def test_2download_one(self):
		"""Download: grab a file."""

		# Initialize a HTTP server
		self.__start_http()

		d = Download.Download(info="quiet", name="Test2")
		filename = "pkg-example_1.%d_all.deb" % self.size_list[0]
		self.__createFile (filename, self.size_list[0])
		d.get(self.makeUrl(0, filename), filename)
		d.wait_mine()
		self.failUnlessEqual(os.path.getsize(filename), os.path.getsize(self.aux_file(filename)),
							 "Downloaded file of wrong size.")
		del d
		self.__removeFiles()

	def test_3download_a_non_existing_file(self):
		"""Download: grab a non existing file."""

		# Initialize a HTTP server
		self.__start_http()

		d = Download.Download(info="quiet", name="Test3")
		filename = "pkg-example-invalid.%d_all.deb" % self.size_list[0]
		#self.__createFile (filename, self.size_list[0])
		d.get(self.makeUrl(0, filename), filename)
		d.wait_mine()
		self.failIf(os.path.exists(filename),
					"Downloaded file even if was not exsistent on remote server.")
		del d

	def test_4dowload_more_file(self):
		"""Download: multiple file download"""
		# Initialize a HTTP server
		self.__start_http()
		self.__start_http()
		self.__start_http()

		d = Download.Download(info="quiet",name= "Test 4")
		self.__createFiles()
		for f in range (0,len(self.size_list)):
			localname = "pkg-example_1.%d_all.deb" % self.size_list[f]
			d.get(self.makeUrl(f, localname), localname)
		d.wait_mine()

		for f in range (0,len(self.size_list)):
			localname = "pkg-example_1.%d_all.deb" % self.size_list[f]
			filename = self.aux_file(localname)
			self.failUnlessEqual(os.path.getsize(filename), os.path.getsize(localname),
								 "Downloaded file has wrong size.")
		del d
		self.__removeFiles()

	def makeUrl(self, portIndex, filename):
		return "http://localhost:%d/%s/%s" % (
			self.running_servers[portIndex],
			self.aux_rel_path(),
			filename
		)

def suite():
	suite = unittest.TestSuite()

	suite.addTest(unittest.makeSuite(DownloadTests, 'test'))

	return suite

if __name__ == '__main__':
	log = logging.getLogger()
	log_handler = logging.FileHandler(sys.argv[0][:-3] + '.log')
	log.setLevel(logging.DEBUG)
	log.addHandler(log_handler)
	unittest.main(defaultTest='suite')



# debpartial_mirror - partial debian mirror package tool
# (c) 2004 Otavio Salvador <otavio@debian.org>, Marco Presi <zufus@debian.org>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA	02111-1307	USA

from cdd import FileSystem

from debpartial_mirror import Config
from debpartial_mirror import Dists
from debpartial_mirror import Files
from debpartial_mirror import Pool

class Backend:
	"""
	This class provides methods to create backendss dirs into the
	partial-mirror
	"""

	backends = []

	def __init__ (self, name, config):
		# Store myself on list
		self.backends.append(self)

		self._cfg = config
		self._name = name

		self._filesystem = FileSystem.FileSystem(self["mirror_dir"])

	def __getitem__ (self, key):
		try:
			item = self._cfg.get_option(key, self._name)
		except Config.InvalidOption, msg:
			raise Config.InvalidOption(self._name, key, "lookup")
		return item

	def has_key (self, key):
		try:
			self.__getitem__ (key)
		except:
			return False
		else:
			return True

	def get_config_with_default(self, key, default):
		if self.has_key(key):
			return self[key]
		return default

	def get_binary_list (self, architecture):
		"""
		Return the partial binList associated to this Backend
		"""
		return self._dists.get_binary_list(architecture)

	def get_source_list (self):
		"""
		Return the partial srcList associated to this Backend
		"""
		return self._dists.get_source_list()

	def get_full_binary_list (self, architecture):
		"""
		Return the full binList associated to this Backend
		"""
		return self._dists.get_full_binary_list(architecture)

	def get_full_source_list (self):
		"""
		Return the full srcList associated to this Backend
		"""
		return self._dists.get_full_source_list()

	def filter(self):
		return self._dists.filter()

	def remove (self):
		""" Remove backend """
		self._pool.remove()
		self._dists.remove()

	def clean (self):
		self._pool.clean()


class MirrorBackend(Backend):
	def __init__ (self, name, config):
		Backend.__init__(self, name, config)
		self._dists = Dists.RemoteDists(self)
		self._pool = Pool.RemotePool(self)
		self._files = Files.RemoteFiles(self)

	def update (self):
		if not self._filesystem.exists(self._name):
			self._filesystem.mkdir(self._name)
		self._dists.update()

	def load (self):
		return self._dists.load()

	def process (self):
		self._dists.process()

	def upgrade (self):
		self._files.upgrade()
		self._pool.upgrade()
		if self.isIndexGenerationRequired():
			self._dists.writeIndexFiles()

	def isIndexGenerationRequired(self):
		return self._pool.containsForeignPackages()


class MergeBackend(Backend):
	def __init__ (self, name, config):
		Backend.__init__(self, name, config)
		self._dists = Dists.MergeDists(self)
		self._pool = Pool.MergePool(self)
		self._files = Files.RemoteFiles(self)

	def merge (self):
		self._dists.merge()
		self._pool.merge()

	def get_mirrors (self):
		return self._dists.get_mirrors()

	def get_packages_for_mirror(self, mirror, architecture):
		return self._dists.get_packages_for_mirror(mirror, architecture)




# debpartial_mirror - partial debian mirror package tool
# (c) 2004 Otavio Salvador <otavio@debian.org>, Nat Budin <natb@brandeis.edu>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA	02111-1307	USA

import logging
import os.path
import ConfigParser
import re
import string


class ConfigException(Exception):
	def getErrorMessage(self):
		return "Unknwon configuration error: %s" % self.__class__.__name__


class InternalError(ConfigException):
	def __init__(self, reason):
		self.reason = reason

	def getErrorMessage(self):
		return "You have found a bug, please report it: %s" % self.reason
		
		
class InvalidOption(ConfigException):
	"""
	Exception called when a invalid option is found in configuration
	file.
	"""
	def __init__(self, section, option, reason):
		self.section = section
		self.option = option
		self.reason = reason

	def getErrorMessage(self):
		return "Wrong option [%s] found in [%s] section : %s" % (
			self.option,
			self.section,
			self.reason,
		)
		

class RequiredOptionMissing(ConfigException):
	"""
	Exception called when a required option in the config file is not
	present.
	"""
	def __init__(self, section, option):
		self.section = section
		self.option = option

	def getErrorMessage(self):
		return 	"Required option [%s] not found in [%s] section." % (self.option, self.section)

		
class InvalidSection(ConfigException):
	"""
	Exception called when a invalid section is found in configuration
	file.
	"""
	def __init__(self, section, reason):
		self.section = section
		self.reason = reason

	def getErrorMessage(self):
		return "Invalid section [%s] : %s" % (self.section, self.reason)

		
class ConfigSection:
	_required = []
	_allowed = []
	_options_with_type = {}
	_allowed_in_filter_field = [
		'subsection',
		'priority',
		'name',
		'exclude-priority',
		'exclude-subsection',
		'exclude-name',
		'include-from',
		'exclude-from',
		'include-script',
		'exclude-script',
		'field-.*',
		'exclude-field-.*',
	]
	_name = ""

	def __init__(self, config, section):
		self.config = config
		self.section = section
		self._allowed_in_filter_regexp = re.compile("|".join(self._allowed_in_filter_field))

		# fix up self._allowed to include all required options too
		for item in self._required:
			if item not in self._allowed:
				self._allowed.append(item)

		self.confs = {}

		for item, value in self.config.items(self.section):
			value = self.__cast_option(item, value)
			if item not in self._allowed:
				rv = self.__parse_variable_options(item)
				if rv is None:
					raise InvalidOption(self.section, item, "%s section" % self._name)
					
				allowed_key, variable, match = rv
				if variable == 'BACKEND':
					if match not in self.config.sections():
						raise InvalidOption(
							self.section, item,
							"[%s] is not the name of a backend" % match,
						)					
				else:
					raise InternalError("[%s] matches unknown variable [%s]" % (item, variable))
					
			self.confs[item] = value

		for item in self._required:
			if not self.confs.has_key(item):
				rv = self.__parse_variable_options(item)
				if rv is None:
					raise RequiredOptionMissing(self.section, item)

	def __parse_variable_options(self, item):
		allowedRe = re.compile("(.*)@(.*)@(.*)")

		for allowed_key in self._allowed:
			allowedMatch = allowedRe.match(allowed_key)
			if allowedMatch is None:
				continue
			before, variable, after = allowedMatch.groups()
			itemMatch = re.match("%s(.*)%s" % (before, after), item)
			if itemMatch is not None:
				return allowed_key, variable, itemMatch.group(1)
		return None

	def __cast_option(self, option, value):
		if option not in self._options_with_type:
			try:
				allowed_key, variable, match = self.__parse_variable_options(option)
				option = allowed_key
			except TypeError:
				pass
		if option in self._options_with_type:
			if self._options_with_type[option] == 'list':
				return value.split()
			elif self._options_with_type[option] == 'boolean':
				value = string.lower(value)
				if value in ('0', 'false', 'no', 'off'):
					return False
				elif value in ('1', 'true', 'yes', 'on'):
					return True
				else:
					raise InvalidOption(self.section, option, "'%s' is not a valid boolean value" % value)
			elif self._options_with_type[option] == 'filter':
				opts = value.split()
				ret = {}
				for opt in opts:
					key, val = opt.split(':', 1)
					if not re.match(self._allowed_in_filter_regexp, key):
						raise InvalidOption(self.section, option, "[%s] is not a filter field" % key)
					if key in ('include-from', 'exclude-from'):
						if not os.path.exists(val):
							raise InvalidOption(self.section, option, "file [%s] doesn't exist" % val) 
					if ret.has_key(key):
						raise InvalidOption(self.section, option, "[%s] option has repeated entries." % key) 

					ret[key] = val
				return ret
		else:
			return value


class ConfigGlobal(ConfigSection):
	_required = [
		'mirror_dir',
		'architectures',
		'components',
		'distributions',
		'get_suggests',
		'get_recommends',
		'get_provides',
		'get_sources',
		'get_packages',
	]

	_allowed = [
		'debug',
		'standalone',
		'display', # WARN: this keyword has not been implemented yet
	]

	_options_with_type = {
		'architectures': 'list',
		'distributions': 'list',
		'get_provides': 'boolean',
		'get_recommends': 'boolean',
		'get_suggests': 'boolean',
		'get_sources': 'boolean',
		'get_packages': 'boolean',
		'components': 'list',
		'standalone' : 'boolean',
	}

	_name = "global"


class ConfigBackendMirror(ConfigSection):
	_allowed = [
		'server',
		'architectures',
		'components',
		'distributions',
		'filter',
		'files',
		'get_suggests',
		'get_recommends',
		'get_provides',
		'get_sources',
		'get_packages',
		'resolve_deps_using',
		'lock',
		'standalone',
		'signature_key',
	]

	_options_with_type = {
		'architectures': 'list',
		'distributions': 'list',
		'filter': 'filter',
		'files': 'list',
		'get_provides': 'boolean',
		'get_recommends': 'boolean',
		'get_suggests': 'boolean',
		'get_sources': 'boolean',
		'get_packages': 'boolean',
		'components': 'list',
		'resolve_deps_using': 'list',
		'lock': 'boolean',
		'standalone' : 'boolean',
	}

	_name = "mirror backend"


class ConfigBackendMerge(ConfigSection):
	_allowed = [
		'filter_@BACKEND@',
		'backends',
		'name',
		'sources_only',
		'origin',
		'label',
		'suite',
		'codename',
		'version',
		'description',
		'signature_key',
		'standalone',
	]

	_options_with_type = {
		'backends': 'list',
		'filter_@BACKEND@': 'filter',
		'standalone' : 'boolean',
	}

	_name = "merge backend"


class Config(ConfigParser.ConfigParser):
	"""
	Store the configurations used by our system.
	"""

	def __init__(self, filename):
		ConfigParser.ConfigParser.__init__(self)
		self.read(filename)

		self.confs = {}
		self.section_objs = {}

		self.backends = {}

		for section in self.sections():
			section_type = self.__get_section_type(section)
			sectionObj = section_type(self, section)
			self.section_objs[section] = sectionObj
			self.confs[section] = sectionObj.confs
			self.confs[section]['name'] = section

		for section in self.section_objs.values():
			if not isinstance(section, ConfigGlobal):
				self.backends[section.section] = section

		# Check backend dependencies
		for backend in self.backends.keys():
			self.check_dependencies(backend)

	def __get_section_type(self, section):
		# detect which config type this is
		if section == 'GLOBAL':
			return ConfigGlobal
		elif 'backends' in self.options(section):
			return ConfigBackendMerge
		elif 'server' in self.options(section):
			return ConfigBackendMirror
		else:
			raise InvalidSection(section, "Unknown section type")

	def get_dependencies(self, backend):
		"""
		Get the list of backends that the given backend depends on
		"""
		if isinstance(self.get_backend(backend), ConfigBackendMirror):
			keyword = 'resolve_deps_using'
		elif isinstance(self.get_backend(backend), ConfigBackendMerge):
			keyword = "backends"
		else:
			raise InvalidSection(backend, "Don't know how to resolve dependencies")
		try:
			dependencies = self.get_option(keyword, backend)
			return dependencies
		except InvalidOption:
			dependencies = []
			return dependencies

	def check_dependencies(self, backend):
		"""
		Checks whether the given beckend depends on valid backend names
		"""
		dependencies = self.get_dependencies(backend)
		for dependency in dependencies:
			if dependency not in self.backends.keys():
				raise InvalidSection(dependency, "missing repository")

	def get_backends(self):

		# Sort backends
		unsorted = self.backends.values()
		sorted = []
		names  = []

		while not len(unsorted) == 0:
			backend = unsorted.pop(0)
			name = backend.section

			dependencies = self.get_dependencies(name)

			if len(dependencies) == 0:
				sorted.append(backend)
				names.append(name)
				continue

			deps_ok = True

			for dependency in dependencies:
				if dependency not in names:
					deps_ok = False
					break

			if deps_ok:
				sorted.append(backend)
				names.append(backend.section)
			else:
				unsorted.append(backend)

		mirrors = []
		merges	= []

		for backend in sorted:
			if isinstance(self.get_backend(backend.section), \
						  ConfigBackendMirror):
				mirrors.append(backend)
			elif isinstance(self.get_backend(backend.section), \
							ConfigBackendMerge):
				merges.append(backend)

		return (mirrors, merges)

	def get_backend(self, name):
		if name in self.section_objs:
			return self.section_objs[name]
		raise InvalidSection(name, "no such backend")

	def get_option(self, option, section='GLOBAL'):
		# specified, fall back to GLOBAL
		if self.confs[section].has_key(option):
			return self.confs[section][option]

		if section != 'GLOBAL':
			logging.debug("[%s] is not present in section [%s]." \
						  "Fallback to global section." % (option, section))
			try:
				return self.get_option(option, 'GLOBAL')
			except InvalidOption, msg:
				raise InvalidOption(section, option, "not found even in global")
			except InvalidSection, msg:
				raise InvalidSection(msg.section)
		else:
			raise InvalidOption(section, option, "not found")

		return self.confs[section][option]


	def dump(self):
		for section, options in self.confs.items():
			print '\n' + section
			for item, value in options.items():
				print "	 %s = %s" % (item, self.get_option(item, section))




import gtk


def on_pop_up_update_activate(popup, app):
    """ Execute when pop-up Update item is selected"""
    print "Update backend pressed", app

def on_pop_up_upgrade_activate (popup, app):
    """ Execute when pop-up Upgrade item is selected """
    print "Upgrade backend pressed", app

def on_b_update_clicked (button, app):
    """ Executed when Update toolbar button is clicked"""
    print "Update selected backends clicked", app
    

def do_popup(app, event):
    """open popup """
    print "Opening popup"
    
    menu = app.xml.get_widget ("backend_pop_up_menu")
    menu.connect ("deactivate", gtk.Widget.hide)
    gtk.Menu.popup(menu, None, None, None, event.button, event.time)

def on_backend_treeview_event(b_treeview, event, app):
    """Check for events """
    if event.type == gtk.gdk.BUTTON_PRESS and event.button == 3:
        do_popup (app, event)

    return False

def backend_toggled_cb (toggle, path, app):
    """ Change the toggle status of the selected backend.
    If a class of backends is choosen (mirrors, merges,..) it
    toggles all the backends of that category

    toggle: is the toggle widget that has been toggled
    path:   represent the item path
    app:    main app

    """
    status = toggle.get_active()
    iter = app.model.get_iter (path)
    
    app.model.set (iter,
                    app.COLUMN_SEL, (not status))
    
    if app.model.iter_has_child (iter):
        child = app.model.iter_children (iter)
        while (isinstance(child, gtk.TreeIter)):
            app.model.set (child, app.COLUMN_SEL, (not status))
            child = app.model.iter_next(child)
            
            app.b_treeview.set_model (app.model)
            

#Toolbar

def quit ():
    gtk.main_quit()    

def on_new_activate (obj, app):
    """Exit the application """
    print app
    print "File->New called"

def on_open_activate (obj, app):
    """ Open a configuration file """
    print "File->Open called"
    open_dialog = gtk.FileChooserDialog("test", action=gtk.FILE_CHOOSER_ACTION_OPEN,
                                        buttons=(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL, gtk.STOCK_OPEN, gtk.RESPONSE_ACCEPT))
    
    r = open_dialog.run()
    if r == gtk.RESPONSE_ACCEPT:
        print open_dialog.get_filename(), "selected"
        del app.conffile
        del app.cnf
        app.conffile = open_dialog.get_filename()
        app.parse_conffile ()
        app.update_backends_treeview ()
    elif r == gtk.RESPONSE_CANCEL:
        pass
    open_dialog.destroy()


def on_save_activate (obj, app):
    """Exit the application """
    print app
    print "File->Save called"


def on_clean_all_activate (obj, app):
    """Exit the application """
    print app
    print "File->Clean called"

    
def on_quit_activate (app):
    """Exit the application """
    print app
    print "File->Exit called"
    quit()


def backend_update (button, app):
    """ Update all selected backends """




# debpartial_mirror - partial debian mirror package tool
# (c) 2004 Otavio Salvador <otavio@debian.org>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA	02111-1307	USA

# TODO: Build two classes to be used. One for remote files and other
# for local files.

import os.path

from cdd import FileSystem

from debpartial_mirror import Download
from debpartial_mirror import DisplayStatus


class Files:
	"""
	This class provides methods to manage normal files of partial-mirrors.

	It can be used to get files based on regexp and full paths.
	"""

	def __init__ (self, backend):
		self._backend = backend
		self._files = []
		self._filesystem = FileSystem.FileSystem(os.path.join(
			backend["mirror_dir"], backend["name"]
		))

	def upgrade(self):
		"""
		This method is used to implement specific logic.

		You MUST to implement it.
		"""
		raise NotImplementedError


class RemoteFiles(Files):
	def upgrade(self):
		download = Download.Download(name = "Files_" + self._backend["name"])
		if self._backend.has_key('files'):
			for file in self._backend['files']:
				download.get(
					self._backend['server'] + '/' + file,
					self._backend['mirror_dir'] + self._backend['name'] + '/' + file
				)
			download.wait_mine()




# debpartial_mirror - partial debian mirror package tool
# (c) 2004 Otavio Salvador <otavio@debian.org>, Enrico Zini <enrico@debian.org>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA	02111-1307	USA

import fcntl
import os.path
import struct
import sys
import termios


class StatusItem:
	id = 0
	def __init__(self):
		self._properties = None

	def __getitem__(self, key):
		if key not in ['id', 'finished', 'current', 'size', 'errored']:
			raise KeyError("%s is not allowed on StatusItem." % key)

		if self._properties is None:
			StatusItem.id += 1
			self._properties = {'id'	  : StatusItem.id,
								'finished': False,
								'current' : 0,
								'size'	  : 0,
								'errored' : ''}

		return self._properties.get(key)

	def __setitem__(self, key, value):
		if key not in ['id', 'finished', 'current', 'size', 'errored']:
			raise KeyError("%s is not allowed on StatusItem." % key)

		if self._properties is None:
			StatusItem.id += 1
			self._properties = {'id'	  : StatusItem.id,
								'finished': False,
								'current' : 0,
								'size'	  : 0,
								'errored' : ''}

		self._properties[key] = value


class BaseDisplayStatus:
	_items = {}

	def __getitem__ (self, key):
		return self._items.get(key, None)

	def start(self, url, size):
		self._items[url] = StatusItem()
		self._items[url]['size'] = size
		self._update()

	def update(self, url, current):
		self._items[url]['current'] = current
		self._update()

	def errored(self, url, message):
		if not self._items.has_key(url):
			self._items[url] = StatusItem()
		self._items[url]['errored'] = message
		self._update()

	def _update(self):
		pass

	def clean(self):
		for i in self._items.keys():
			self._items.pop(i)


class TextDisplayStatus(BaseDisplayStatus):
	def __print_status(self):
		self.__clear_line()
		for url in self._items.keys():
			if self._items[url]['finished'] or self._items[url]['errored']:
				continue
			try:
				sys.stdout.write("[%d %s: %.2f%%]"
								 % (self._items[url]['id'],
									os.path.basename(url).split('_')[0],
									self._items[url]['current']*100/
									(self._items[url]['size'])))
			except ZeroDivisionError:
				pass
		sys.stdout.flush()

	def __clear_line(self):
		if sys.stdout.isatty():
			sw = struct.unpack(
				"hhhh",
				fcntl.ioctl(1, termios.TIOCGWINSZ, " "*8)
			)[1]
			sys.stdout.write("\r" + " " * sw + "\r")

	def start(self, url, size):
		BaseDisplayStatus.start(self, url, size)
		self.__clear_line()
		sys.stdout.write("\rGetting %d: %s\n" % (self._items[url]['id'], url))
		sys.stdout.flush()

	def update(self, url, current):
		BaseDisplayStatus.update(self, url, current)
		self.__clear_line()
		if current >= self._items[url]['size'] and \
			   not self._items[url]['finished']:
			sys.stdout.write("\rDone: %s\n" % url)
			self._items[url]['finished'] = True
		self.__print_status()

	def errored(self, url, message):
		if not self._items.has_key(url) or not self._items[url]['errored']:
			BaseDisplayStatus.errored(self, url, message)
			self.__clear_line()
			sys.stdout.write("\rFailed: %s %s\n" % (url, message))
			sys.stdout.flush()


class LogDisplayStatus(BaseDisplayStatus):

	def start(self, url, size):
		BaseDisplayStatus.start(self, url, size)
		print "Getting %d: %s" % (self._items[url]['id'], url)

	def update(self, url, current):
		BaseDisplayStatus.update(self, url, current)
		if current >= self._items[url]['size'] and \
			   not self._items[url]['finished']:
			self._items[url]['finished'] = True

	def errored(self, url, message):
		if not self._items.has_key(url) or not self._items[url]['errored']:
			BaseDisplayStatus.errored(self, url, message)
			print "Failed: %s %s\n" % (url, message)


class QuietDisplayStatus(BaseDisplayStatus):
	def start(self, url, size):
		pass

	def update(self, url, current):
		pass

	def errored(self, url, message):
		if not self._items.has_key(url) or not self._items[url]['errored']:
			BaseDisplayStatus.errored(self, url, message)
			print "Failed: %s %s\n" % (url, message)


class GtkDisplayStatus(BaseDisplayStatus):
	#Same as TextDisplayStatus, but send data to a widget instead of print
	pass



# debpartial_mirror - partial debian mirror package tool
# (c) 2004 Otavio Salvador <otavio@debian.org>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA	02111-1307	USA

import StringIO
import threading
import Queue
import re
import os
import os.path
from stat import *
import pycurl
from string import split, join
from rfc822 import formatdate
from errno import EEXIST

from debpartial_mirror import DisplayStatus


class DownloadQueue(Queue.Queue):
	""" Implement a Queue without duplicated items. """

	counter = 0

	def _put(self, item):
		if item not in self.queue:
			self.queue.append(item)


class DownloadFileHandler:
	def __init__(self, filename, mode, buffering = 0):
		self._filename = filename
		self._mode = mode
		self._buffering = buffering
		self._openned = False
		self.__fp = None

	def write(self, string):
		if not self._openned:
			self.__fp = open(self._filename, self._mode, self._buffering)
			self._openned = True

		self.__fp.write(string)

	def close(self):
		if self.__fp:
			self.__fp.close()
			self.__fp = None
			self._openned = False



class Curl:
	buggy_curl = False

	def __init__(self, DisplayStatus, Queue):
		# Handle bug version of pycurl (versions pior 7.14 are buggy)
		major, minor, revision = pycurl.version_info()[1].split('.')
		if (int(major) + 0.01*int(minor) < 7.14) and not Curl.buggy_curl:
			print "WARNING: Due a bug in libcurl up to 7.13 we can't use resume support!"
			Curl.buggy_curl = True

		self._curl = pycurl.Curl()
		self._curl.setopt(pycurl.FOLLOWLOCATION, 1)
		self._curl.setopt(pycurl.MAXREDIRS, 5)
		self._curl.setopt(pycurl.NOSIGNAL, 0)
		self._curl.setopt(pycurl.CONNECTTIMEOUT, 30)
		self._curl.setopt(pycurl.PROGRESSFUNCTION, self.progress)
		self._curl.setopt(pycurl.NOPROGRESS, 0)
		self._curl.setopt(pycurl.FAILONERROR, 1)
		self._curl.setopt(pycurl.HTTP_VERSION, pycurl.CURL_HTTP_VERSION_1_0)
		self._curl.setopt(pycurl.VERBOSE, 0)

		self._curl.parent = self
		self._curl.callback = None
		self._curl.error_callback = None
		self._curl.filename = None

		self._fp = None
		self._startSize = 0
		self.DisplayStatus = DisplayStatus
		self.Queue = Queue

	def __processDir(self):
		# in case it's a directory, we process to find all needed
		# objects and queue all.
		if self._fp != None and isinstance(self._fp, StringIO.StringIO):
			self._fp.seek(0)
			matches = re.compile(
				'<a href="([a-zA-Z0-9_][a-zA-Z0-9\-\./_]+)">',
				 re.IGNORECASE | re.MULTILINE
			).findall(self._fp.read())

			for filen in matches:
				try:
					os.makedirs(os.path.dirname(self._curl.filename))
				except OSError, (errno, msg):
					if errno != EEXIST:
						print "ERROR:", msg
				# (uri, destine, callback, error_callback, no_cache)
				self.Queue.put((
					os.path.dirname(self._curl.url) + '/' + filen,
					os.path.dirname(self._curl.filename) + '/' + filen,
					self._curl.callback, self._curl.error_callback,
					self._curl.no_cache
				))

	def set_target(self, url, filename, callback, error_callback, no_cache):
		self._curl.setopt(pycurl.URL, url)
		self._curl.url = url
		self._curl.callback = callback
		self._curl.error_callback = error_callback
		self._curl.filename = filename
		self._curl.setopt(pycurl.RESUME_FROM, 0)
		self._curl.no_cache = no_cache

		if self._fp is not None:
			try:
				self._fp.close()
				del self._fp # don't leave the files open
			except IOError:
				self._fp = None

		extra_header = []
		if filename.endswith('/'):
			self._fp = StringIO.StringIO()
		else:
			self._curl.filename += ".partial"

			# We'll rename it back if is the same file
			if os.path.exists (filename):
				extra_header.append("if-modified-since:" + formatdate(
					os.path.getmtime(filename)
				))
				os.rename (filename, self._curl.filename)

			if not Curl.buggy_curl and os.path.exists (self._curl.filename):
				#dowload interrupted, needs resume
				self._startSize = os.stat(self._curl.filename)[ST_SIZE]
				self._curl.setopt(pycurl.RESUME_FROM, self._startSize)
				self._fp = DownloadFileHandler(self._curl.filename, "ab")
			else :
				self._fp = DownloadFileHandler(self._curl.filename, "wb")

		self._curl.setopt(pycurl.WRITEFUNCTION, self._fp.write)


		if not no_cache:
			extra_header.append("Pragma:")

		if extra_header:
			self._curl.setopt(pycurl.HTTPHEADER, extra_header)

	def close(self):
		self.__processDir()
		self.callback()
		self._fp = None
		self._curl.close()

	def progress(self, download_t, download_d, upload_t, upload_d):
		#If we doesn't know how much data we will receive, return.
		if download_t == 0 or download_d == 0:
		   return

		if self.DisplayStatus[self._curl.url] == None and self._curl.url[-1] != '/':
			self.DisplayStatus.start(self._curl.url, download_t + self._startSize)

		if self._curl.url[-1] != '/':
			self.DisplayStatus.update(self._curl.url, download_d + self._startSize)

	def callback(self):
		if not self._curl.filename:
			return # It hasn't set!
		#remove trailing .partial after download is completed
		if not isinstance(self._fp, StringIO.StringIO) and os.path.exists (self._curl.filename):
			self._fp.close()
			os.rename (self._curl.filename, split (self._curl.filename,".partial")[0])
		else:
			self.__processDir()

		#exec user passed callback
		if self._curl.callback != None:
			self._curl.callback
			self._curl.callback = None

	def error_callback(self):
		if self._curl.error_callback != None:
			self._curl.error_callback
			self._curl.error_callback = None


class DownloadFetcher(threading.Thread):
	def __init__(self, info, downloaders):
		# Add DisplayStatus object.
		# TODO: the DisplayStatus type should be set in the configuration file
		if info == "text":
			self.DisplayStatus = DisplayStatus.TextDisplayStatus()
		elif info == "log":
			self.DisplayStatus = DisplayStatus.LogDisplayStatus()
		elif info == "quiet":
			self.DisplayStatus = DisplayStatus.QuietDisplayStatus()
		else:
			self.DisplayStatus = info

		# Create the needed pycurl objects to manage the connections.
		self._multi = pycurl.CurlMulti()
		self._multi.handles = []
		self._free = Queue.Queue()
		self.queue = DownloadQueue()
		for i in range(downloaders):
			curl = Curl(self.DisplayStatus, self.queue)
			self._multi.handles.append(curl)

		self.running = False
		map(lambda x: self._free.put(x), self._multi.handles)

		threading.Thread.__init__(self)

	def start(self):
		self.running = True
		threading.Thread.start(self)

	def run(self):
		total_handles = 0;
		while self.running:
			Download.lock.acquire()
			while self.queue.qsize() > 0:
				try:
					fetcher = self._free.get_nowait()
					url, filename, callback, error_callback, no_cache = self.queue.get_nowait()
					fetcher.set_target(url, filename, callback, error_callback, no_cache)
					self._multi.add_handle(fetcher._curl)
					total_handles += 1
				except Queue.Empty:
					break
			Download.lock.release()

			#If all fetcher are free, we finished our job
			if self._free.qsize() == len (self._multi.handles):
				break

			# Run the internal curl state machine for the multi stack
			ret = self._multi.select(1.0)
			if ret == -1:
				print "Select returned -1"
				break

			while 1:
				ret, num_handles = self._multi.perform()

				if num_handles < total_handles:
					#One or more fetcher finished download, we call
					#respective curl callbacks, and free the fetcher itself"
					num_failed, ok_list, err_list = self._multi.info_read()
					for curl in ok_list:
						curl.parent.callback()
						self._multi.remove_handle(curl)
						self._free.put(curl.parent)
					for curl, errno, errmsg in err_list:
						self.DisplayStatus.errored(curl.url, errmsg)
						curl.parent.error_callback()
						self._multi.remove_handle(curl)
						self._free.put(curl.parent)
					#update the number of actual fetcher
					total_handles = num_handles
				if ret != pycurl.E_CALL_MULTI_PERFORM:
					break

		Download.lock.acquire()
		while self.queue.qsize()>0:
			self.queue.get_nowait()
		Download.lock.release()
		try:
			while len(self._multi.handles) > 0:
				curl = self._multi.handles.pop()
				try:
					self._multi.remove_handle(curl._curl)
				except:
					pass
				curl.close()
				del curl
				self._multi.close()
		except:
			pass
		self.running = False
		self.DisplayStatus.clean()


class Download:
	""" A Download queue """
	# Fetcher to use
	d_fetchers_list = {}

	# Our lock handler
	lock = threading.Lock()

	def __init__(self, info="text", max=3, name=None):
		self.__max = max
		self.__info = info
		self.__name = name
		self.d_fetcher = None
		if self.__name == None:
			self.__name = str(self).split(" ")[-1].rstrip(">")
		if not isinstance(self.__name,str):
			raise TypeError, "The name passed, was not a string"
		elif self.__name in Download.d_fetchers_list:
			raise KeyError, "A Downloaded fetcher with name '%s' alrady exists" % self.__name

	def set_name (name):
		"""Set a name for the queue"""
		print "This function should not be called"
		if not self.__name:
			self.__name = name

	def request (self, uri, destine, callback = None, error_callback = None, no_cache = False):
		"""Add an uri to the queue. The uri is copied on the local path destine.
		   Optionally, it is possible to pass two callbacks functions that are
		   executed once download is terminated (callback) or if it failed (error_callback)"""
		# Create the needed d_fetcher.
		self.lock.acquire()
		if self.d_fetcher == None or not self.d_fetcher.running:
			self.d_fetcher = DownloadFetcher(self.__info, self.__max)
			self.d_fetcher.setDaemon(True)
			self.d_fetcher.start()
			Download.d_fetchers_list[self.__name]= self.d_fetcher
		self.d_fetcher.queue.put((uri, destine, callback, error_callback, no_cache))
		self.lock.release()

	def get (self, uri, destine, callback = None, error_callback = None, no_cache = False):
		self.request (uri, destine, callback, error_callback, no_cache)

	def get_name (self):
		"""Return the name of this queue"""
		return self.__name

	def get_names (self):
		"""Return a list the current queues"""
		return Download.d_fetchers_list.keys()

	def wait(self, name):
		"""Wait for all files in the queue 'name' to be downloaded """
		try:
			self._wait(Download.d_fetchers_list[name])
			self.lock.acquire()
			Download.d_fetchers_list.pop(name)
			d_fetcher = None
			self.lock.release()
		except KeyError:
			print "No thread with given name"

	def _wait(self, d_fetcher):
		d_fetcher.join(0.5)
		while d_fetcher and d_fetcher.running:
			d_fetcher.join(0.5)

	def wait_all(self):
		"""Wait for all the files in all the queues are downloaded."""
		# We need to use timeout to handle signals
		while 1:
			try:
				i = Download.d_fetchers_list.iterkeys()
				n = i.next()
				f = Download.d_fetchers_list[n]
				f.running = False
				self.wait(n)
			except StopIteration:
				break

	def wait_mine(self):
		"""Wait for all files in this queue to be downloaded."""
		self.wait(self.__name)



import os
import unittest

import mock

from TestBase import TestBase
from RepositorySimulator import PooledRepository, NonPooledRepository

from debpartial_mirror import Backend
from debpartial_mirror import Config
from debpartial_mirror import Controller
from debpartial_mirror import Dists
from debpartial_mirror import Download
from debpartial_mirror import Failures


class EndToEndTests(TestBase):
	def setUp(self):
		TestBase.setUp(self)
		self.mockCommandExecutor = mock.Mock()
		Dists.Dists._commandExecutor = self.mockCommandExecutor.execute


	def test01a(self):
		""" Mirror a pooled repository - basic """
		sourceRepository = PooledRepository()
		self.addPackages(sourceRepository, ["package1", "package2"], distribution="etch", component="main", architecture="i386")
		self.addInstallerPackages(sourceRepository, ["udeb1", "udeb2"], distribution="etch", component="main", architecture="i386")
		sourceDir = self.writeRepository(sourceRepository)
		globalConfiguration, mirrorDir = self.createGlobalConfiguration(
			components="main main/debian-installer",
		)
		configuration = self.loadConfiguration(globalConfiguration + """
[myRepo]
server = file://%s
"""% sourceDir)

		self.doAll(configuration)

		mirroredRepository = self.readMirror(PooledRepository, mirrorDir,multi=True)
		self.checkPackages(
			["package1", "package2"],
			mirroredRepository,
			distribution="etch", component="main", architecture="i386",
		)
		self.checkInstallerPackages(
			["udeb1", "udeb2"],
			mirroredRepository,
			distribution="etch", component="main", architecture="i386"
		)


	def test01b(self):
		""" Mirror a pooled repository - partial archs"""
		sourceRepository = PooledRepository()
		self.addPackages(sourceRepository, ["package1"], distribution="etch", component="main", architecture="i386")
		self.addPackages(sourceRepository, ["package2"], distribution="etch", component="main", architecture="alpha")
		self.addPackages(sourceRepository, ["package3"], distribution="etch", component="main", architecture="arm")
		sourceDir = self.writeRepository(sourceRepository)

		globalConfiguration, mirrorDir = self.createGlobalConfiguration(
			architectures = "i386 arm",
		)
		configuration = self.loadConfiguration(globalConfiguration + """
[myRepo]
server = file://%s
"""% sourceDir)

		self.doAll(configuration)

		mirroredRepository = self.readMirror(PooledRepository, mirrorDir,multi=True)
		self.checkPackages(["package1"], mirroredRepository, distribution="etch", component="main", architecture="i386")
		self.checkPackages([], mirroredRepository, distribution="etch", component="main", architecture="alpha")
		self.checkPackages(["package3"], mirroredRepository, distribution="etch", component="main", architecture="arm")


	def test01c(self):
		""" Mirror a pooled repository - partial components"""
		sourceRepository = PooledRepository()
		self.addPackages(sourceRepository, ["package1"], distribution="etch", component="main", architecture="i386")
		self.addPackages(sourceRepository, ["package2"], distribution="etch", component="contrib", architecture="i386")
		self.addPackages(sourceRepository, ["package3"], distribution="etch", component="non-free", architecture="i386")
		sourceDir = self.writeRepository(sourceRepository)

		globalConfiguration, mirrorDir = self.createGlobalConfiguration(
			components = "main contrib",
		)
		configuration = self.loadConfiguration(globalConfiguration + """
[myRepo]
server = file://%s
"""% sourceDir)

		self.doAll(configuration)

		mirroredRepository = self.readMirror(PooledRepository, mirrorDir,multi=True)
		self.checkPackages(["package1"], mirroredRepository, distribution="etch", component="main", architecture="i386")
		self.checkPackages(["package2"], mirroredRepository, distribution="etch", component="contrib", architecture="i386")
		self.checkPackages([], mirroredRepository, distribution="etch", component="non-free", architecture="i386")


	def test01d(self):
		""" Mirror a pooled repository - partial distributions"""
		sourceRepository = PooledRepository()
		self.addPackages(sourceRepository, ["package1"], distribution="etch", component="main", architecture="i386")
		self.addPackages(sourceRepository, ["package2"], distribution="lenny", component="main", architecture="i386")
		self.addPackages(sourceRepository, ["package3"], distribution="sid", component="main", architecture="i386")
		sourceDir = self.writeRepository(sourceRepository)
		globalConfiguration, mirrorDir = self.createGlobalConfiguration(
			distributions = "etch lenny",
		)
		configuration = self.loadConfiguration(globalConfiguration + """
[myRepo]
server = file://%s
"""% sourceDir)

		self.doAll(configuration)

		mirroredRepository = self.readMirror(PooledRepository, mirrorDir,multi=True)
		self.checkPackages(["package1"], mirroredRepository, distribution="etch", component="main", architecture="i386")
		self.checkPackages(["package2"], mirroredRepository, distribution="lenny", component="main", architecture="i386")
		self.checkPackages([], mirroredRepository, distribution="sid", component="main", architecture="i386")


	def test01e(self):
		""" Mirror a pooled repository - same package in multiple archs"""
		sourceRepository = PooledRepository()
		self.addPackages(sourceRepository, ["package1"], distribution="etch", component="main", architecture="i386")
		self.addPackages(sourceRepository, ["package1", "package3"], distribution="etch", component="main", architecture="arm")
		self.addPackages(sourceRepository, ["package2"], distribution="etch", component="main", architecture="alpha")
		sourceDir = self.writeRepository(sourceRepository)

		globalConfiguration, mirrorDir = self.createGlobalConfiguration(
			architectures = "i386 arm",
		)
		configuration = self.loadConfiguration(globalConfiguration + """
[myRepo]
server = file://%s
"""% sourceDir)

		self.doAll(configuration)

		mirroredRepository = self.readMirror(PooledRepository, mirrorDir,multi=True)
		self.checkPackages(["package1"], mirroredRepository, distribution="etch", component="main", architecture="i386")
		self.checkPackages([], mirroredRepository, distribution="etch", component="main", architecture="alpha")
		self.checkPackages(["package1", "package3"], mirroredRepository, distribution="etch", component="main", architecture="arm")


	def test01f(self):
		""" Mirror a pooled repository : keep latest version per architecture """
		sourceRepository = PooledRepository()

		self.addPackages(sourceRepository, ["package1_1.0", "package1_1.1"], distribution="etch", component="main", architecture="i386")
		self.addPackages(sourceRepository, ["package1_1.2"], distribution="etch", component="main", architecture="arm")
		sourceDir = self.writeRepository(sourceRepository)

		globalConfiguration, mirrorDir = self.createGlobalConfiguration(
			architectures = "i386 arm",
		)
		configuration = self.loadConfiguration(globalConfiguration + """
[myRepo]
server = file://%s
"""% sourceDir)

		self.doAll(configuration)

		mirroredRepository = self.readMirror(PooledRepository, mirrorDir,multi=True)
		#self.failUnlessEqual(("1.0", "1.1"), sourceRepository.getAvailableVersions("package1",  distribution="etch", component="main", architecture="i386"))
		self.failUnlessEqual(("1.1", ), mirroredRepository.getAvailableVersions("package1",  distribution="etch", component="main", architecture="i386"))

		self.failUnlessEqual(("1.2", ), mirroredRepository.getAvailableVersions("package1",  distribution="etch", component="main", architecture="arm"))


	def test02a(self):
		""" Mirror a pooled repository - filters """
		sourceRepository = PooledRepository()
		self.addPackages(sourceRepository, ["package1", "package2", "stuff1", "stuff2"], distribution="etch", component="main", architecture="i386")
		sourceDir = self.writeRepository(sourceRepository)
		globalConfiguration, mirrorDir = self.createGlobalConfiguration()
		configuration = self.loadConfiguration(globalConfiguration + """
[myRepo]
server = file://%s
filter = name:.*1
"""% sourceDir)

		self.doAll(configuration)

		mirroredRepository = self.readMirror(PooledRepository, mirrorDir)
		self.checkPackages(
			["package1", "stuff1"],
			mirroredRepository,
			distribution="etch", component="main", architecture="i386"
		)


	def test02b(self):
		""" Mirror a pooled repository - filters with dependencies"""
		sourceRepository = PooledRepository()
		packageList = self.addPackages(sourceRepository,
			["skipped", "depends1", "suggests1", "recommends1", "requiredButExcluded"],
			distribution="etch", component="main", architecture="i386"
		)
		packageList.addPackage("required1", Depends="depends1", Suggests="suggests1", Recommends="recommends1")
		sourceDir = self.writeRepository(sourceRepository)
		globalConfiguration, mirrorDir = self.createGlobalConfiguration()
		configuration = self.loadConfiguration(globalConfiguration + """
[myRepo]
server = file://%s
filter = name:required.* exclude-name:.*Excluded
"""%  sourceDir)

		self.doAll(configuration)

		mirroredRepository = self.readMirror(PooledRepository, mirrorDir)
		self.checkPackages(
			#FIXME: not actually implemented...
			#["required1", "depends1", "recommends1"],
			["required1", "depends1"],
			mirroredRepository,
			distribution="etch", component="main", architecture="i386"
		)


	def test02c(self):
		""" Mirror a pooled repository - filters with unmet dependencies (non standalone)"""
		sourceRepository = PooledRepository()
		packageList = self.addPackages(sourceRepository, ["skipped", "depends1", "depends3"], distribution="etch")
		packageList.addPackage("required1", Depends="depends1, depends2, depends3")
		sourceDir = self.writeRepository(sourceRepository)
		globalConfiguration, mirrorDir = self.createGlobalConfiguration(
			get_recommends = "true",
		)
		configuration = self.loadConfiguration(globalConfiguration + """
[myRepo]
server = file://%s
filter = name:required.*
"""%  sourceDir)

		self.doAll(configuration)

		mirroredRepository = self.readMirror(PooledRepository, mirrorDir)
		self.checkPackages(
			["required1", "depends1", "depends3"],
			mirroredRepository,
			distribution="etch", component="main", architecture="i386"
		)


	def test02d(self):
		""" Mirror a pooled repository - filters with unmet dependencies (standalone)"""
		sourceRepository = PooledRepository()
		packageList = self.addPackages(sourceRepository,
			["skipped", "depends1", "depends3"], distribution="etch"
		)
		packageList.addPackage("required1", Depends="depends1, depends2, depends3, depends4")
		sourceDir = self.writeRepository(sourceRepository)
		globalConfiguration, mirrorDir = self.createGlobalConfiguration()
		configuration = self.loadConfiguration(globalConfiguration + """
[myRepo]
server = file://%s
filter = name:required.*
standalone = true
"""%  sourceDir)

		try :
			self.doAll(configuration)
			self.fail("Exception not raised")
		except Failures.DependencyResolutionFailure, ex:
			print ex.format()
			self.failUnlessEqual("required1 needs [depends2,depends4]", ex.format())


	def test02e(self):
		""" Mirror a pooled repository - filters with architecture specific dependencies"""
		sourceRepository = PooledRepository()
		packageList1 = self.addPackages(sourceRepository,
			["depends1",],
			distribution="etch", component="main", architecture="i386"
		)
		packageList2 = self.addPackages(sourceRepository,
			["depends2",],
			distribution="etch", component="main", architecture="arm"
		)
		packageList1.addPackage("required1", Depends="depends1")
		packageList2.addPackage("required2", Depends="depends2")
		sourceDir = self.writeRepository(sourceRepository)
		globalConfiguration, mirrorDir = self.createGlobalConfiguration(architectures="i386 arm")
		configuration = self.loadConfiguration(globalConfiguration + """
[myRepo]
server = file://%s
filter = name:required.* exclude-name:.*Excluded

"""%  sourceDir)

		self.doAll(configuration)

		mirroredRepository = self.readMirror(PooledRepository, mirrorDir, multi=True)
		self.checkPackages(
			["required1", "depends1"],
			mirroredRepository,
			distribution="etch", component="main", architecture="i386"
		)
		self.checkPackages(
			["required2", "depends2"],
			mirroredRepository,
			distribution="etch", component="main", architecture="arm"
		)

	def test03a(self):
		""" Mirror a source repositroy - basic """
		sourceRepository = PooledRepository()
		# FIXME: pure source repositories not supported even if get_packages=false
		self.addPackages(sourceRepository, ["package1", "package2"], distribution="etch", component="main", architecture="i386")
		packageList = sourceRepository.createSourcePackageList(distribution="etch", component="main")
		packageList.addPackage("srcPackage1")
		packageList.addPackage("srcPackage2")
		sourceDir = self.writeRepository(sourceRepository)
		globalConfiguration, mirrorDir = self.createGlobalConfiguration(
			get_packages = "false",
			get_sources = "true",
		)
		configuration = self.loadConfiguration(globalConfiguration + """
[myRepo]
server = file://%s
"""% sourceDir)

		self.doAll(configuration)

		mirroredRepository = self.readMirror(PooledRepository, mirrorDir)
		return # FIXME: source packages not currently fetched!!
		self.checkSourcePackages(
			["srcPackage1", "srcPackage2"],
			mirroredRepository,
			distribution="etch", component="main",
		)


	def test04a(self):
		""" Merge - ensure all package fields copied """
		sourceRepository = PooledRepository()
		packageList = sourceRepository.createPackageList(distribution="etch", component="main", architecture="i386")
		fields = {
			"Essential":"yes",
			"Priority":"required",
			"Section":"shells",
			"Installed-Size":"1999",
			"Maintainer":"a.n other",
			"Replaces":"old stuff",
			"Depends":"depends1",
			"Pre-Depends":"predepends1",
			"Suggests":"suggests1",
			"Recommends":"recommends1",
			"Size":"123456",
			"MD5sum":"md5 sum",
			"SHA1":"sha1 sum",
			"SHA256":"sha256 sum",
			"Description":"Short description\n line1\ line2",
			"Tag":"tag1",
		}
		packageList.addPackage("package1", **fields)
		packageList.addPackage("depends1")
		packageList.addPackage("suggests1")
		packageList.addPackage("recommends1")

		sourceDir = self.writeRepository(sourceRepository)
		globalConfiguration, mirrorDir = self.createGlobalConfiguration()
		configuration = self.loadConfiguration(globalConfiguration + """
[src1]
server = file://%s

[myRepo]
backends = src1
"""% sourceDir)

		self.doAll(configuration)

		mergedRepository = self.readMirror(PooledRepository, mirrorDir)

		package = mergedRepository.getPackage("package1", distribution="myRepo", component="main", architecture="i386")
		for key, expectedValue in fields.iteritems():
			self.failUnlessEqual(expectedValue, package[key])


	def test04b(self):
		""" Merge two pooled repositories - basic with default signature"""
		sourceRepository1 = PooledRepository()
		self.addPackages(sourceRepository1, ["A", "C"], distribution="etch", component="main", architecture="i386")
		sourceDir1 = self.writeRepository(sourceRepository1)
		sourceRepository2 = PooledRepository()
		self.addPackages(sourceRepository2, ["B", "D"], distribution="etch", component="main", architecture="i386")
		sourceDir2 = self.writeRepository(sourceRepository2)

		globalConfiguration, mirrorDir = self.createGlobalConfiguration()
		configuration = self.loadConfiguration(globalConfiguration + """
[src1]
server = file://%s

[src2]
server = file://%s

[myRepo]
backends = src1 src2
signature_key = default
"""% (sourceDir1, sourceDir2))

		self.mockCommandExecutor.execute.return_value = 0
		self.doAll(configuration)
		self.mockCommandExecutor.execute.assert_called_with(
			"gpg -abs  -o - %s/myRepo/dists/myRepo/Release > %s/myRepo/dists/myRepo/Release.gpg" % (mirrorDir, mirrorDir)
		)
		mergedRepository = self.readMirror(PooledRepository, mirrorDir)
		self.checkPackages(
			["A", "B", "C", "D"],
			mergedRepository,
			distribution="myRepo", component="main", architecture="i386"
		)


	def test04c(self):
		""" Merge two pooled repositories - debian installer and specific signature"""
		sourceRepository1 = PooledRepository()
		self.addInstallerPackages(sourceRepository1, ["udebA", "udebC"], distribution="etch", component="main", architecture="i386")
		sourceDir1 = self.writeRepository(sourceRepository1)
		sourceRepository2 = PooledRepository()
		self.addInstallerPackages(sourceRepository2, ["udebB", "udebD"], distribution="etch", component="main", architecture="i386")
		sourceDir2 = self.writeRepository(sourceRepository2)

		globalConfiguration, mirrorDir = self.createGlobalConfiguration(
			components = "main/debian-installer",
		)
		configuration = self.loadConfiguration(globalConfiguration + """
[src1]
server = file://%s

[src2]
server = file://%s

[myRepo]
backends = src1 src2
signature_key = DEADBEEF
"""% (sourceDir1, sourceDir2))

		self.mockCommandExecutor.execute.return_value = 0
		self.doAll(configuration)
		self.mockCommandExecutor.execute.assert_called_with(
			"gpg -abs -u DEADBEEF -o - %s/myRepo/dists/myRepo/Release > %s/myRepo/dists/myRepo/Release.gpg" % (mirrorDir, mirrorDir)
		)
		mergedRepository = self.readMirror(PooledRepository, mirrorDir)
		self.checkInstallerPackages(
			["udebA", "udebB", "udebC", "udebD"],
			mergedRepository,
			distribution="myRepo", component="main", architecture="i386"
		)


	def test04d(self):
		""" Merge from a dual part components (etch/updates) """
		sourceRepository = PooledRepository()
		self.addPackages(sourceRepository, ["A", "B"], distribution="etch", component="updates/main", architecture="i386")

		sourceDir = self.writeRepository(sourceRepository)
		globalConfiguration, mirrorDir = self.createGlobalConfiguration(
			components = "updates/main",
		)
		configuration = self.loadConfiguration(globalConfiguration + """
[src1]
server = file://%s

[myRepo]
backends = src1
"""% sourceDir)

		self.doAll(configuration)

		mergedRepository = self.readMirror(PooledRepository, mirrorDir)
		self.checkPackages(
			["A", "B"],
			mergedRepository,
			distribution="myRepo", component="updates/main", architecture="i386"
		)


	def test05a(self):
		""" Merge with filter """
		sourceRepository1 = PooledRepository()
		self.addPackages(sourceRepository1, ["inc1", "exc1", "inc2", "exc2"], distribution="etch")
		sourceDir1 = self.writeRepository(sourceRepository1)
		sourceRepository2 = PooledRepository()
		packageList = self.addPackages(sourceRepository2, ["got1", "exc3", "got2", "exc4"], distribution="etch")
		packageList.addPackage("depends1")
		packageList.addPackage("suggests1")
		packageList.addPackage("recommends1")
		packageList.addPackage("gotWithDeps", Depends="depends1", Suggests="suggests1", Recommends="recommends1")

		sourceDir2 = self.writeRepository(sourceRepository2)

		globalConfiguration, mirrorDir = self.createGlobalConfiguration()
		configuration = self.loadConfiguration(globalConfiguration + """
[src1]
server = file://%s

[src2]
server = file://%s

[myRepo]
backends = src1 src2
filter_src1 = name:inc
filter_src2 = name:got
"""% (sourceDir1, sourceDir2))

		self.doAll(configuration)

		mergedRepository = self.readMirror(PooledRepository, mirrorDir)
		self.checkPackages(
			["depends1", "got1", "got2", "gotWithDeps", "inc1", "inc2"],
			mergedRepository,
			distribution="myRepo", component="main", architecture="i386"
		)


	def test06a(self):
		""" Mirror a non pooled repository - basic """
		sourceRepository = NonPooledRepository()
		self.addPackages(sourceRepository, ["package1", "package2"], distribution="etch", component="main", architecture="i386")
		sourceDir = self.writeRepository(sourceRepository)
		globalConfiguration, mirrorDir = self.createGlobalConfiguration()
		configuration = self.loadConfiguration(globalConfiguration + """
[myRepo]
server = file://%s
"""% sourceDir)

		self.doAll(configuration)

		mirroredRepository = self.readMirror(NonPooledRepository, mirrorDir)
		self.checkPackages(["package1", "package2"], mirroredRepository, distribution="etch", component="main", architecture="i386")


	def test06b(self):
		""" Merge two non pooled repositories """
		sourceRepository1 = NonPooledRepository()
		self.addPackages(sourceRepository1, ["A", "C"], distribution="etch", component="main", architecture="i386")
		sourceDir1 = self.writeRepository(sourceRepository1)
		sourceRepository2 = NonPooledRepository()
		self.addPackages(sourceRepository2, ["B", "D"], distribution="etch", component="main", architecture="i386")
		sourceDir2 = self.writeRepository(sourceRepository2)

		globalConfiguration, mirrorDir = self.createGlobalConfiguration()
		configuration = self.loadConfiguration(globalConfiguration + """
[src1]
server = file://%s

[src2]
server = file://%s

[myRepo]
backends = src1 src2
"""% (sourceDir1, sourceDir2))

		self.doAll(configuration)

		mergedRepository = self.readMirror(NonPooledRepository, mirrorDir)
		self.checkPackages(
			["A", "B", "C", "D"],
			mergedRepository,
			distribution="myRepo", component="main", architecture="i386"
		)


	def test07a(self):
		""" Mirror update on filter change """
		sourceRepository = PooledRepository()
		packageList = self.addPackages(sourceRepository, ["skipped1", "skipped2", "depends1"], distribution="etch")

		packageList.addPackage("required1", Depends="depends1")
		sourceDir = self.writeRepository(sourceRepository)
		globalConfiguration, mirrorDir = self.createGlobalConfiguration()
		baseConfigFile = globalConfiguration +"""
[myRepo]
server = file://%s
"""% sourceDir
		self.doAll(self.loadConfiguration(baseConfigFile))

		mirroredRepository = self.readMirror(PooledRepository, mirrorDir)
		self.checkPackages(
			["required1", "depends1", "skipped1", "skipped2"],
			mirroredRepository,
			distribution="etch", component="main", architecture="i386"
		)
		print 80*"="
		Download.Curl.buggy_curl = True  # seems to have problems on my machine even though OK...
		self.doAll(self.loadConfiguration(baseConfigFile + "filter = name:required.*"))
		mirroredRepository = self.readMirror(PooledRepository, mirrorDir)
		self.checkPackages(
			["required1", "depends1"],
			mirroredRepository,
			distribution="etch", component="main", architecture="i386"
		)

		print 80*"="
		self.doAll(self.loadConfiguration(baseConfigFile))

		mirroredRepository = self.readMirror(PooledRepository, mirrorDir)
		self.checkPackages(
			["required1", "depends1", "skipped1", "skipped2"],
			mirroredRepository,
			distribution="etch", component="main", architecture="i386"
		)


	def test07b(self):
		""" Merge mirror update on filter change """
		sourceRepository = PooledRepository()
		packageList = self.addPackages(sourceRepository, ["skipped1", "skipped2", "depends1"], distribution="etch")

		packageList.addPackage("required1", Depends="depends1")
		sourceDir = self.writeRepository(sourceRepository)
		globalConfiguration, mirrorDir = self.createGlobalConfiguration()
		baseConfigFile = globalConfiguration +"""
[myRepo]
backends = source

[source]
server = file://%s
"""% sourceDir
		self.doAll(self.loadConfiguration(baseConfigFile))

		mirroredRepository = self.readMirror(PooledRepository, mirrorDir)
		self.checkPackages(
			["required1", "depends1", "skipped1", "skipped2"],
			mirroredRepository,
			distribution="myRepo", component="main", architecture="i386"
		)
		print 80*"="
		Download.Curl.buggy_curl = True  # seems to have problems on my machine even though OK...
		self.doAll(self.loadConfiguration(baseConfigFile + "filter = name:required.*"))
		mirroredRepository = self.readMirror(PooledRepository, mirrorDir)
		self.checkPackages(
			["required1", "depends1"],
			mirroredRepository,
			distribution="myRepo", component="main", architecture="i386"
		)
		print 80*"="
		self.doAll(self.loadConfiguration(baseConfigFile))

		mirroredRepository = self.readMirror(PooledRepository, mirrorDir)
		self.checkPackages(
			["required1", "depends1", "skipped1", "skipped2"],
			mirroredRepository,
			distribution="myRepo", component="main", architecture="i386"
		)


	def test08a(self):
		""" Dependency resolution from another repository (resolve_deps_using) """
		sourceRepository = PooledRepository(missingReleaseFields=("Version",))
		packageList = self.addPackages(sourceRepository, ["pack1", "pack2",], distribution="etch")
		packageList.addPackage("pack3", Depends="dep1, dep2")
		depRepository = PooledRepository()
		self.addPackages(depRepository, ["dep1", "dep2", "other1"], distribution="etch")
		sourceDir = self.writeRepository(sourceRepository)
		depDir = self.writeRepository(depRepository)

		globalConfiguration, mirrorDir = self.createGlobalConfiguration()
		configuration = self.loadConfiguration(globalConfiguration + """
[dep]
server = file://%s

[myRepo]
server = file://%s
resolve_deps_using = dep
signature_key = default
"""% (depDir, sourceDir))

		self.mockCommandExecutor.execute.return_value = 0
		self.doAll(configuration)
		self.mockCommandExecutor.execute.assert_called_with(
			"gpg -abs  -o - %s/myRepo/dists/etch/Release > %s/myRepo/dists/etch/Release.gpg" % (mirrorDir, mirrorDir)
		)

		myRepository = self.readMirror(PooledRepository, mirrorDir)
		self.checkPackages(
			["pack1", "pack2", "pack3", "dep1", "dep2"],
			myRepository,
			distribution="etch", component="main", architecture="i386"
		)

		for field, value in sourceRepository.getReleaseFields().iteritems():
			if not field in ("Date",):
				self.failUnlessEqual(value, myRepository.getReleaseFields()[field])

#TODO: trivial repositories



	def createGlobalConfiguration(self, **kw):
		configMap = {
			"architectures" :"i386",
			"components" : "main",
			"distributions" : "etch",
			"get_suggests" : "false",
			"get_recommends" : "false",
			"get_provides" : "false",
			"get_sources" : "false",
			"get_packages" : "true",
		}
		configMap.update(kw)
		mirrorDir = self.createTempDirectory()
		configMap["mirror_dir"] = mirrorDir

		configLines = ["[GLOBAL]"]
		for key, value in configMap.iteritems():
			configLines.append("%s = %s" % (key, value))
		configLines.append("")
		return "\n".join(configLines),  mirrorDir

	def readMirror(self, class_, mirrorDir, name="myRepo", multi=False):
		mirroredRepository = class_(multi=multi)
		mirroredRepository.readFromFilesystem(os.path.join(mirrorDir, name))
		return mirroredRepository


	def addPackages(self, repository, packageNames, distribution, component="main", architecture="i386"):
		return self._addPackages(repository.createPackageList(distribution, component, architecture), packageNames)

	def addInstallerPackages(self, repository, packageNames, distribution, component="main", architecture="i386"):
		return self._addPackages(repository.createInstallerPackageList(distribution, component, architecture), packageNames)

	def _addPackages(self, packageList, packageNames):
		for name in packageNames:
			packageList.addPackage(name)
		return packageList

	def writeRepository(self, repository):
		directory = os.path.abspath(self.createTempDirectory()) # always absolute for use in file:// urls
		repository.writeToFilesystem(directory)
		return directory

	def loadConfiguration(self, configString):
		configFile = self.createTempFile()
		f = open(configFile, "w")
		try:
			f.write(configString)
		finally:
			f.close()
		return Config.Config(configFile)

	def doAll(self, configuration):
		Backend.Backend.backends = [] #TODO: fix static
		Download.Download.d_fetchers_list = {}
		controller = Controller.Controller(configuration, [])
		controller.executeCommand("all")


	def checkPackages(self, expectedPackageNames, repository, distribution, component="main", architecture="i386"):
		self.failUnlessEqual(
			sorted(expectedPackageNames),
			sorted(repository.getAvailablePackageNames(distribution, component, architecture)),
		)
		if not repository.isMulti(): # Cannot tell which files for which arch,component etc just by looking at filenames
			self.failUnlessEqual(
				sorted(expectedPackageNames),
				sorted(repository.getPackageNamesForPresentFiles()),
			)

	def checkInstallerPackages(self, expectedPackageNames, repository, distribution, component="main", architecture="i386"):
		self.failUnlessEqual(
			sorted(expectedPackageNames),
			sorted(repository.getAvailableInstallerPackageNames(distribution, component, architecture)),
		)

	def checkSourcePackages(self, expectedPackageNames, repository, distribution, component="main"):
		self.failUnlessEqual(
			sorted(expectedPackageNames),
			sorted(repository.getAvailableSourcePackageNames(distribution, component)),
		)




def suite():
	suite = unittest.TestSuite()
	suite.addTest(unittest.makeSuite(EndToEndTests, 'test'))
	return suite


if __name__ == '__main__':
	unittest.main()



class MirrorFailure(Exception):
	pass

class DependencyResolutionFailure(MirrorFailure):
	def __init__(self, badDepsByPackage):
		self._badDepsByPackage = badDepsByPackage
		MirrorFailure.__init__(self)

	def format(self, prefix=None, linePrefix=""):
		lines = []
		if prefix is not None:
			lines.append(prefix)

		def renderDep(dep):
			return "".join(dep)

		def renderAlternatives(deps):
			return "|".join([renderDep(x) for x in deps])

		for packageName, dependencies in self._badDepsByPackage.iteritems():
			lines.append("%s%s needs [%s]" % (linePrefix, packageName, ",".join(
				[renderAlternatives(x) for x in dependencies]
			)))
		return "\n".join(lines)




# debpartial_mirror_ui - User Interface for partial debian mirror package tool
# (c) 2006 Otavio Salvador <otavio@debian.org>
# (c) 2006 Marco Presi <zufus@debian.org>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


#TODO: Create a CellRenderer that contains both the toggle button and the text.



import gobject
from gtk import VBox, Label, TreeView, \
                 TreeIter, TreeViewColumn, TreeStore, \
                 CellRendererToggle, CellRendererText, \
                 gdk, Frame, SHADOW_IN, SELECTION_SINGLE

from debpartial_mirror import Backend
from debpartial_mirror import Config


class BackendBrowser(VBox):
    """ A widget to browse Backend configuration """


    __gsignals__ = dict(backend_changed=(gobject.SIGNAL_RUN_FIRST,
                                         gobject.TYPE_NONE,
                                         (gobject.TYPE_STRING,)))

    
    def __init__(self, cnf, debug=True):
        super(BackendBrowser, self).__init__()

        self._debug_mode = debug
        self.l = Label ('Backend Browser')
        bframe = Frame ()
        
        self.b = TreeView ()

        self.read_backends (cnf)
        
        (self.COLUMN_SEL,
         self.COLUMN_NAME) = range(2)

        self.create_browser (self.b)
        bframe.add (self.b)

        self.pack_start (self.l, False, False, 5)
        self.pack_start (bframe, True)

        self.show_all()



    def _get_backends (self, cnf, type):
        """Get the backend list from config file"""
        backend = []
        for b in cnf.get_backends ()[type]:
            backend.append(Backend.MirrorBackend (b.section, cnf))
        return backend

    def _get_mirrors (self, cnf):
        return self._get_backends (cnf, 0)
    
    def _get_merges (self, cnf):
        return self._get_backends (cnf, 1)

    def read_backends (self, cnf):
        """Read the backends defined in the conf file """
        self.mirrors = self._get_mirrors(cnf)
        self.merges  = self._get_merges(cnf)


    def create_browser (self, b):
        ''' Create a TreeView that allows to browse all the backend
            defined in the Config file'''

        b.set_property ('visible', True)
        b.set_property ('can_focus', True)
        b.set_property ('headers_visible', True)
        b.set_property ('enable_search', True)
        b.set_property ('reorderable', True)
        b.set_property ('rules_hint',False)
        b.set_property ('fixed_height_mode', False)
        b.set_property ('hover_selection', False)
        b.set_property ('hover_expand', False)

        b.connect ("event", self.on_b_event)

        model = TreeStore (gobject.TYPE_BOOLEAN,
                           gobject.TYPE_STRING)

        self.update_backends_treeview (model)
        b.set_model (model)

        #column for backend selection 
        toggle = CellRendererToggle ()
        toggle.set_property ("activatable", True)
        toggle.connect ('toggled', self.backend_toggled_cb, model)

        text = CellRendererText ()
        text.set_property ("xalign", 0.0)
        
        col = TreeViewColumn()

        col.pack_start (toggle, False)
        col.pack_start (text, True)

        col.add_attribute (toggle, 'active', self.COLUMN_SEL)
        col.add_attribute (text, 'markup', self.COLUMN_NAME)

        col.set_clickable (True)
        col.set_sort_column_id (self.COLUMN_NAME)
        b.append_column (col)

        select = b.get_selection ()
        select.set_mode (SELECTION_SINGLE)
        select.connect ('changed', self.backend_selection_changed_cb)
        b.expand_all()
        b.set_rules_hint (True)
        b.set_headers_visible (False)
        b.show_all()

    def update_backends_treeview (self, model):
        """ Update the browser """
        model.clear ()

        global_iter = model.append (None)
        model.set (global_iter,
                   self.COLUMN_NAME, "<b>Global</b>")
        
        mirrors_iter = model.append(None)
        model.set (mirrors_iter,
                   self.COLUMN_SEL, False,
                   self.COLUMN_NAME, "<b>Mirrors</b>")

        merges_iter = model.append(None)
        model.set (merges_iter,
                   self.COLUMN_SEL, False,
                   self.COLUMN_NAME, "<b>Merges</b>")

        for i in self.mirrors:
            mirrors_list_iter = model.append (mirrors_iter)
            model.set (mirrors_list_iter,
                       #self.COLUMN_SEL, None,
                       self.COLUMN_NAME, i._name)

        if len(self.mirrors) == 0:
            mirrors_list_iter = model.append (mirrors_iter)
            model.set (mirrors_list_iter,
                       #self.COLUMN_SEL, None,
                       self.COLUMN_NAME, "<i>No Mirrors defined</i>")


        for i in self.merges:
            merges_list_iter = model.append (merges_iter)
            model.set (merges_list_iter,
                       #self.COLUMN_SEL, None,
                       self.COLUMN_NAME, i._name)
        
        if len(self.merges) == 0:
            merges_list_iter = model.append (merges_iter)
            model.set (merges_list_iter,
                       #self.COLUMN_SEL, None,
                       self.COLUMN_NAME, "<i>No Merges defined</i>")

        

    def backend_toggled_cb (self, toggle, path, model):
        """ Change the toggle status of the selected backend.
        If a class of backends is choosen (mirrors, merges,..) it
        toggles all the backends of that category
        
        toggle: is the toggle widget that has been toggled
        path:   represent the item path
        app:    main app
        
        """
        status = toggle.get_active()
        iter = model.get_iter (path)

        
        model.set (iter, self.COLUMN_SEL, (not status))
        
        if model.iter_has_child (iter):
            child = model.iter_children (iter)
            while (isinstance(child, TreeIter)):
                model.set (child, self.COLUMN_SEL, (not status))
                child = model.iter_next(child)
                
                self.b.set_model (model)


    def backend_selection_changed_cb (self, selection):
        """ When a backend is selected, we show its properties in the property
        treeview."""
        print self
        print selection
        (model, iter) = selection.get_selected ()
        if isinstance (iter, TreeIter):
            print "%s selected\n" % model.get_value (iter, self.COLUMN_NAME)
            self.emit ('backend_changed', model.get_value (iter, self.COLUMN_NAME))
            


    def on_b_event(self, treeview, event):
        if event.type == gdk.BUTTON_PRESS and event.button == 3:
            self.do_popup (event)

    def do_popup(self, event):
        """open popup """
        print "Opening popup"



##########################################################################
#
# __init__.py:  defines this directory as 'debpartial_mirror' package.
#
#  debpartial_mirror is an application to make easier to build full or
#  partial mirrors from Debian or compatible archives.
#  See http://partial-mirror.alioth.debian.org/ for more information.
#    
# ====================================================================
# Copyright (c) 2002-2005 OS Systems.  All rights reserved.
#
# This software is licensed as described in the file COPYING, which
# you should have received as part of this distribution.
#
#########################################################################
# Author: Otavio Salvador <otavio@ossystems.com.br>




# debpartial_mirror_ui - User Interface for partial debian mirror package tool
# (c) 2006 Otavio Salvador <otavio@debian.org>
# (c) 2006 Marco Presi <zufus@debian.org>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import gnome
import gtk.glade
import gobject

from BackendBrowser import BackendBrowser
from ConfigEditor import ConfigEditor

#for debuggin purpose


from debpartial_mirror import Backend
from debpartial_mirror import Config
from debpartial_mirror import Download

from BackendBrowser import BackendBrowser

from Callbacks import *

name               = "gdebpartial-mirror"
version            = "0.0.1"
app_glade_file     = "debpartial_mirror_ui/interface/ui.glade"
backend_glade_file = "debpartial_mirror_ui/interface/backend_ui.glade"
cnf_file           = "/etc/debpartial-mirror.conf"


class GDebPartialMirror:
    """ A class that represent the main interface """
    def __init__ (self, conffile):
        self.app = gnome.program_init (name, version)
        self.cnf = Config.Config (cnf_file)

        gtk.glade.set_custom_handler (self.get_custom_handler)
        self.xml = gtk.glade.XML (app_glade_file)

        self.debug_mode = True
        auto_signals = {"on_new_activate": (on_new_activate, self),
                        "on_open_activate": (on_open_activate, self),
                        "on_save_activate": (on_save_activate, self),
                        "on_clean_all_activate": (on_clean_all_activate, self),
                        "on_quit_activate": on_quit_activate,
                        "on_backend_treeview_event":  (on_backend_treeview_event, self),
                        "on_pop_up_update_activate":  (on_pop_up_update_activate, self),
                        "on_pop_up_upgrade_activate": (on_pop_up_upgrade_activate,self),
                        "on_b_update_clicked":        (on_b_update_clicked, self),
                        "on_backend_selection_change": (self.on_backend_selection_change)}
    
        
#        self.create_backends_treeview ()
        
        self.xml.signal_autoconnect (auto_signals)

        #self.b.connect ('backend_changed', self.c.update_property_editor)

    def get_custom_handler(self, glade, function_name, widget_name,
                           str1, str2, int1, int2):
        """
        Generic handler for creating custom widgets, used to
        enable custom widgets.
        
        The custom widgets have a creation function specified in design time.
        Those creation functions are always called with str1,str2,int1,int2 as
        arguments, that are values specified in design time.
        
        This handler assumes that we have a method for every custom widget
        creation function specified in glade.
        
        If a custom widget has create_foo as creation function, then the
        method named create_foo is called with str1,str2,int1,int2 as arguments.
        """
        handler = getattr(self, function_name)
        return handler(str1, str2, int1, int2)
    
    def create_backend_browser (self, str1, str2, int1, int2):
        self.b = BackendBrowser (self.cnf)
        return self.b
    
    def create_configuration_editor (self, str1, str2, int1, int2):
        self.c = ConfigEditor (self.cnf) 
        return self.c

    def on_backend_selection_change (self, backend, b_name):
        """ When a backend is selected, we show its properties in the property
        treeview."""
        self.c.update_property_editor (b_name)



 # debpartial-mirror - partial debian mirror package tool
# (c) 2004 Otavio Salvador <otavio@debian.org>, Nat Budin <natb@brandeis.edu>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA	02111-1307	USA

import unittest

from TestBase import TestBase
from debpartial_mirror import Config

class ConfigurationFilesTests(TestBase):
	def loadConfig(self, conf_file, exception):
		""" Generic test code to handle configuration file load and
		exception handling for it. """
		try:
			cnf = Config.Config(conf_file)
			return cnf
		except Config.InvalidOption, msg:
			if not exception or not isinstance(msg, exception):
				self.fail("""Wrongly intercept as Config.InvalidOption exception. %s was expected.
				section: %s
				option:	 %s""" % (type(exception), msg.section, msg.option))

		except Config.InvalidSection, msg:
			if not exception or not isinstance(msg, exception):
				self.fail("""Wrongly intercept as Config.InvalidSection exception. %s was expected.
				section: %s""" % (type(exception), msg.section))

		except Config.RequiredOptionMissing, msg:
			if not exception or not isinstance(msg, exception):
				self.fail("""Wrongly intercept as Config.RequiredOptionMissing exception. %s was expected.
				section: %s
				option:	 %s""" % (type(exception), msg.section, msg.option))

		if not exception:
			self.fail("Fail to raise the need exception. %s was expected." % type(exception))

	def testGoodConfigurationFile(self):
		"""Config: valid configuration file"""
		self.loadConfig(self.aux_file('TestConfig-good.conf'), None)

	def testMissingRequiredOption(self):
		"""Config: missing required option"""
		self.loadConfig(self.aux_file('TestConfig-missing-required.conf'), Config.RequiredOptionMissing)

	def testInvalidOption(self):
		"""Config: invalid option"""
		self.loadConfig(self.aux_file('TestConfig-invalid-option.conf'), Config.InvalidOption)

	def testInvalidOptionValue(self):
		"""Config: invalid option value"""
		self.loadConfig(self.aux_file('TestConfig-invalid-option-value.conf'), Config.InvalidOption)

	def testSectionType(self):
		"""Config: invalid option due a section type"""
		self.loadConfig(self.aux_file('TestConfig-section-types.conf'), Config.InvalidOption)

	def testInvalidFilter(self):
		"""Config: invalid filter"""
		self.loadConfig(self.aux_file('TestConfig-invalid-filter.conf'), Config.InvalidOption)

	def testInvalidSection(self):
		"""Config: invalid section"""
		self.loadConfig(self.aux_file('TestConfig-invalid-section.conf'), Config.InvalidSection)

	def testTypeCast(self):
		"""Config: type casting"""
		def castCompare(option, section, right):
			self.failUnless(cnf.get_option(option, section) == right,
							"We were expecting to receive '%s' but we received '%s'."
							% (right, cnf.get_option(option, section)))

		cnf = self.loadConfig(self.aux_file('TestConfig-typecast.conf'), None)

		# List conversion
		castCompare('architectures', 'GLOBAL', ['i386', 'powerpc'])
		castCompare('components', 'GLOBAL', ['main'])
		castCompare('distributions', 'GLOBAL', ['stable'])

		# Boolean conversion
		castCompare('get_suggests', 'GLOBAL', False)
		castCompare('get_recommends', 'GLOBAL', True)
		castCompare('get_provides', 'GLOBAL', True)
		castCompare('get_packages', 'GLOBAL', True)
		castCompare('get_sources', 'GLOBAL', False)

def suite():
	suite = unittest.TestSuite()

	suite.addTest(unittest.makeSuite(ConfigurationFilesTests, 'test'))

	return suite

if __name__ == '__main__':
	import logging, sys
	log = logging.getLogger()
	log_handler = logging.FileHandler(sys.argv[0][:-3] + '.log')
	log.setLevel(logging.DEBUG)
	log.addHandler(log_handler)
	unittest.main(defaultTest='suite')


##########################################################################
#
# setup.py: File to build and install the application.
#
#  debpartial-mirror is an application to make easier to build full or
#  partial mirrors from Debian or compatible archives.
#  See http://partial-mirror.alioth.debian.org/ for more information.
#    
# ====================================================================
# Copyright (c) 2002-2005 OS Systems.  All rights reserved.
#
# This software is licensed as described in the file COPYING, which
# you should have received as part of this distribution.
#
#########################################################################
# Author: Otavio Salvador <otavio@ossystems.com.br>

from os import listdir
from distutils.core import setup

lib_files = []
for f in listdir('debpartial_mirror/'):
    if f[-3:] == '.py':
        lib_files.append(f)

setup(
    name = "debpartial-mirror",
    version = '0.3.0',
    author = "Otavio Salvador",
    author_email = "otavio@debian.org",
    url = "http://partial-mirror.alioth.debian.org",
    license = "GPL",
    description = "tools to create partial Debian mirrors",
    scripts = [ 'debpartial-mirror' ],
    packages = [ 'debpartial_mirror' ]
    )
    

    
    



from debpartial_mirror import Backend


class Controller:
	def __init__(self, configuration, mirrorNames):
		self._mirrors = []
		self._merges = []
		cnf_mirrors, cnf_merges = configuration.get_backends()

		def useBackend(backend):
			if not mirrorNames:
				return True
			return backend.section in mirrorNames

		for b in cnf_mirrors:
			if useBackend(b):
				self._mirrors.append(Backend.MirrorBackend(b.section, configuration))

		for b in cnf_merges:
			if useBackend(b):
				self._merges.append(Backend.MergeBackend(b.section, configuration))

		self._commands = {
			"all" : self.doAll,
			"update" : self.doUpdate,
			"upgrade" : self.doUpgrade,
			"merge" : self.doMerge,
			"clean" : self.doClean,
		}
	def isValidCommand(self, commandName):
		return commandName in self._commands

	def executeCommand(self, commandName):
		self._commands.get(commandName)()

	def doAll(self):
		self._update()
		if self._load():
			self._process()
			self._upgrade()
			self._merge()
			self._clean()

	def doUpdate(self):
		self._update()

	def doUpgrade(self):
		if self._load():
			self._process()
			self._upgrade()

	def doMerge(self):
		if self._load():
			self._process()
			self._merge()

	def doClean(self):
		if self._load():
			self._clean()

	def _update(self):
		for b in self._mirrors:
			if b.has_key('lock') and b['lock']:
				print "Skipping backend", b._name
			else:
				print "Updating backend", b._name
				b.update()

	def _load(self):
		for b in self._mirrors:
			print "Loading backend", b._name
			if not b.load():
				return False
		return True

	def _process(self):
		for b in self._mirrors:
			print "Processing backend", b._name
			b.process()

	def _upgrade(self):
		for b in self._mirrors:
			if b.has_key('lock') and b['lock']:
				print "Skipping backend", b._name
			else:
				print "Upgrading backend", b._name
				b.upgrade()

	def _clean(self):
		for b in self._mirrors + self._merges:
			if b.has_key('lock') and b['lock']:
				print "Skipping backend", b._name
			else:
				print "Clean backend", b._name
				b.clean()

	def _merge(self):
		for b in self._merges:
			print "Merging backend", b._name
			b.merge()

